@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -27,6 +27,8 @@ require 'milter_manager.so'
 
 require 'milter/compatible'
 
+require 'milter/manager/exception'
+
 require 'milter/manager/child-context'
 require 'milter/manager/connection-check-context'
 require 'milter/manager/netstat-connection-checker'
@@ -40,6 +42,9 @@ require 'milter/manager/redhat-detector'
 require 'milter/manager/freebsd-rc-detector'
 require 'milter/manager/pkgsrc-rc-detector'
 
+require 'milter/manager/postfix-cidr-table'
+require 'milter/manager/postfix-regexp-table'
+
 Milter::Logger.domain = "milter-manager"
 
 module Milter::Manager
@@ -97,7 +102,7 @@ module Milter::Manager
       @result << "\n"
 
       dump_item("manager.connection_spec", c.manager_connection_spec.inspect)
-      dump_item("manager.unix_socket_mode", "0%o" % c.manager_unix_socket_mode)
+      dump_item("manager.unix_socket_mode", "%#o" % c.manager_unix_socket_mode)
       dump_item("manager.unix_socket_group", c.manager_unix_socket_group.inspect)
       dump_item("manager.remove_unix_socket_on_create",
                 c.remove_manager_unix_socket_on_create?)
@@ -114,6 +119,13 @@ module Milter::Manager
       dump_item("manager.max_file_descriptors", c.max_file_descriptors.inspect)
       dump_item("manager.custom_configuration_directory",
                 c.custom_configuration_directory.inspect)
+      dump_item("manager.fallback_status", c.fallback_status.nick.dump)
+      dump_item("manager.fallback_status_at_disconnect",
+                c.fallback_status_at_disconnect.nick.dump)
+      dump_item("manager.event_loop_backend",
+                c.event_loop_backend.nick.dump)
+      dump_item("manager.n_workers", c.n_workers)
+      dump_item("manager.packet_buffer_size", c.default_packet_buffer_size)
       dump_item("manager.connection_check_interval",
                 c.connection_check_interval.inspect)
       @result << "\n"
@@ -225,7 +237,7 @@ module Milter::Manager
   end
 
   class ConfigurationLoader
-    class Error < StandardError
+    class Error < Milter::Manager::Error
     end
 
     class InvalidValue < Error
@@ -861,6 +873,53 @@ module Milter::Manager
         @configuration.connection_check_interval = interval
       end
 
+      def event_loop_backend
+        @configuration.event_loop_backend
+      end
+
+      def event_loop_backend=(backend)
+        update_location("event_loop_backend", backend.nil?)
+        @configuration.event_loop_backend = backend
+      end
+
+      def n_workers
+        @configuration.n_workers
+      end
+
+      def n_workers=(n_workers)
+        update_location("n_workers", n_workers.nil?)
+        n_workers ||= 0
+        @configuration.n_workers = n_workers
+      end
+
+      def packet_buffer_size
+        @configuration.default_packet_buffer_size
+      end
+
+      def packet_buffer_size=(size)
+        update_location("size", size.nil?)
+        size ||= 0
+        @configuration.default_packet_buffer_size = size
+      end
+
+      def fallback_status
+        @configuration.fallback_status
+      end
+
+      def fallback_status=(status)
+        update_location("fallback_status", status.nil?)
+        @configuration.fallback_status = status
+      end
+
+      def fallback_status_at_disconnect
+        @configuration.fallback_status_at_disconnect
+      end
+
+      def fallback_status_at_disconnect=(status)
+        update_location("fallback_status_at_disconnect", status.nil?)
+        @configuration.fallback_status_at_disconnect = status
+      end
+
       def define_connection_checker(name, &block)
         old_id = @connection_checker_ids[name]
         @configuration.signal_handler_disconnect(old_id) if old_id
