@@ -1,6 +1,6 @@
 /* -*- c-file-style: "ruby" -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -17,15 +17,66 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "../../../../config.h"
+#  undef PACKAGE_NAME
+#  undef PACKAGE_TARNAME
+#  undef PACKAGE_VERSION
+#  undef PACKAGE_STRING
+#  undef PACKAGE_BUGREPORT
+#endif /* HAVE_CONFIG_H */
+
 #include "rb-milter-client-private.h"
 
 #define SELF(self) RVAL2GOBJ(self)
 
 VALUE rb_cMilterClient;
 
+static GPid
+client_custom_fork (MilterClient *client)
+{
+#ifdef HAVE_RB_FORK
+    int status;
+    return (GPid)rb_fork(&status, NULL, NULL, Qnil);
+#else
+    VALUE pid;
+
+    pid = rb_funcall2(rb_mKernel, rb_intern("fork"), 0, 0);
+    if (NIL_P(pid)) {
+	return (GPid)0;
+    } else {
+	return (GPid)NUM2INT(pid);
+    }
+#endif
+}
+
+static VALUE
+client_initialize (VALUE self)
+{
+    MilterClient *client;
+
+    client = milter_client_new();
+    G_INITIALIZE(self, client);
+    milter_client_set_custom_fork_func(client, client_custom_fork);
+    return Qnil;
+}
+
+static VALUE
+client_run (VALUE self)
+{
+    GError *error = NULL;
+
+    if (!milter_client_run(SELF(self), &error))
+	RAISE_GERROR(error);
+
+    return Qnil;
+}
+
 static VALUE
 client_main (VALUE self)
 {
+    milter_warning("[ruby][client][deprecatd] Milter::Client#main is deprecated. "
+		   "Use Milter::Client#run instead.");
     milter_client_main(SELF(self));
     return Qnil;
 }
@@ -44,6 +95,23 @@ client_start_syslog (VALUE self, VALUE identify)
     return Qnil;
 }
 
+static VALUE
+client_listen (VALUE self)
+{
+    GError *error = NULL;
+
+    if (!milter_client_listen(SELF(self), &error))
+	RAISE_GERROR(error);
+
+    return Qnil;
+}
+
+static VALUE
+client_create_context (VALUE self)
+{
+    return GOBJ2RVAL_UNREF(milter_client_create_context(SELF(self)));
+}
+
 static VALUE
 client_get_connection_spec (VALUE self)
 {
@@ -161,6 +229,19 @@ client_set_default_unix_socket_mode (VALUE self, VALUE rb_mode)
     return Qnil;
 }
 
+static VALUE
+client_get_default_packet_buffer_size (VALUE self)
+{
+    return UINT2NUM(milter_client_get_default_packet_buffer_size(SELF(self)));
+}
+
+static VALUE
+client_set_default_packet_buffer_size (VALUE self, VALUE size)
+{
+    milter_client_set_default_packet_buffer_size(SELF(self), NUM2UINT(size));
+    return Qnil;
+}
+
 static void
 mark (gpointer data)
 {
@@ -189,9 +270,14 @@ Init_milter_client (void)
 		    "DEFAULT_MAX_CONNECTIONS",
 		    UINT2NUM(MILTER_CLIENT_DEFAULT_MAX_CONNECTIONS));
 
+    rb_define_method(rb_cMilterClient, "initialize", client_initialize, 0);
+    rb_define_method(rb_cMilterClient, "run", client_run, 0);
     rb_define_method(rb_cMilterClient, "main", client_main, 0);
     rb_define_method(rb_cMilterClient, "shutdown", client_shutdown, 0);
     rb_define_method(rb_cMilterClient, "start_syslog", client_start_syslog, 1);
+    rb_define_method(rb_cMilterClient, "listen", client_listen, 0);
+    rb_define_method(rb_cMilterClient, "create_context",
+		     client_create_context, 0);
     rb_define_method(rb_cMilterClient, "connection_spec",
                      client_get_connection_spec, 0);
     rb_define_method(rb_cMilterClient, "set_connection_spec",
@@ -216,8 +302,18 @@ Init_milter_client (void)
                      client_get_default_unix_socket_mode, 0);
     rb_define_method(rb_cMilterClient, "set_default_unix_socket_mode",
                      client_set_default_unix_socket_mode, 1);
+    rb_define_method(rb_cMilterClient, "default_packet_buffer_size",
+                     client_get_default_packet_buffer_size, 0);
+    rb_define_method(rb_cMilterClient, "set_default_packet_buffer_size",
+                     client_set_default_packet_buffer_size, 1);
 
     G_DEF_SETTERS(rb_cMilterClient);
 
+    G_DEF_CLASS(MILTER_TYPE_CLIENT_EVENT_LOOP_BACKEND,
+		"ClientEventLoopBackend",
+		rb_mMilter);
+    G_DEF_CONSTANTS(rb_cMilterClient, MILTER_TYPE_CLIENT_EVENT_LOOP_BACKEND,
+		    "MILTER_CLIENT_");
+
     Init_milter_client_context();
 }
