@@ -19,6 +19,8 @@
 
 #include "rb-milter-core-private.h"
 
+VALUE rb_mMilterCore;
+
 void
 Init_milter_core (void)
 {
@@ -41,4 +43,5 @@ Init_milter_core (void)
     Init_milter_error_emittable();
     Init_milter_agent();
     Init_milter_protocol_agent();
+    Init_milter_event_loop();
 }
