@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2009-2010  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -512,7 +512,6 @@ EOC
                   "unix:/var/run/spamass.sock"]])
   end
 
-
   def test_apply_enma_style
     enma_conf = @tmp_dir + "enma.conf"
     (@init_d + "enma").open("w") do |file|
@@ -545,12 +544,42 @@ EOC
                   "inet:10025@127.0.0.1"]])
   end
 
+  def test_apply_opendkim_style
+    opendkim_conf = @tmp_dir + "opendkim.conf"
+    (@init_d + "opendkim").open("w") do |file|
+      file << opendkim_init_header(opendkim_conf.to_s)
+    end
+
+    opendkim_conf.open("w") do |conf|
+      conf << <<-EOC
+# Basic OpenDKIM config file
+# See opendkim.conf(5) or /usr/share/doc/opendkim/opendkim.conf.sample for more
+Mode   v
+Syslog  yes
+Socket local:/var/run/opendkim/opendkim.socket
+EOC
+    end
+
+    detector = redhat_init_detector("opendkim")
+    detector.detect
+    detector.apply(@loader)
+    assert_equal("opendkim", detector.name)
+    assert_eggs([["opendkim",
+                  "OpenDKIM milter for signing and verifying email",
+                  false,
+                  (@init_d + "opendkim").to_s,
+                  "start",
+                  "local:/var/run/opendkim/opendkim.socket"]])
+  end
+
   private
-  def redhat_init_detector(name)
+  def redhat_init_detector(name, options={})
     detector = Milter::Manager::RedHatInitDetector.new(@configuration, name)
 
     _init_base_dir = @init_base_dir
     _etc_dir = @etc_dir
+    _candidate_service_commands = options[:candidate_service_commands] || []
+
     singleton_object = class << detector; self; end
     singleton_object.send(:define_method, :init_base_dir) do
       _init_base_dir.to_s
@@ -560,6 +589,10 @@ EOC
       _etc_dir.to_s
     end
 
+    singleton_object.send(:define_method, :candidate_service_commands) do
+      _candidate_service_commands
+    end
+
     detector
   end
 
@@ -813,6 +846,40 @@ prog=enma
 ENMA=/usr/libexec/enma
 CONF_FILE=#{enma_conf}
 LOCK_FILE=/var/lock/subsys/enma
+EOS
+  end
+
+  def opendkim_init_header(opendkim_conf)
+    <<-EOS
+#!/bin/sh
+#
+# $Id: opendkim.init,v 1.1 2009/08/13 08:54:09 mmarkley Exp $
+#
+#### BEGIN INIT INFO
+# Provides:          opendkim
+# Required-Start:    $syslog $time $local_fs $remote_fs $named $network
+# Required-Stop:     $syslog $time $local_fs $remote_fs $named
+# Default-Start:     2 3 4 5
+# Default-Stop:      S 0 1 6
+# Short-Description: OpenDKIM Milter
+# Description:       The OpenDKIM milter for signing and verifying email
+#                    messages using the DomainKeys Identified Mail protocol
+### END INIT INFO
+#
+# chkconfig: 345 20 80
+# description: OpenDKIM milter for signing and verifying email
+# processname: opendkim
+#
+# This script should run successfully on any LSB-compliant system. It will
+# attempt to fall back to RedHatisms if LSB isn't available, or to
+# commonly-available utilities if it's not even RedHat.
+
+NAME=opendkim
+PATH=/bin:/usr/bin:/sbin:/usr/sbin
+DAEMON=/usr/sbin/$NAME
+PIDFILE=/var/run/$NAME/$NAME.pid
+CONFIG=#{opendkim_conf}
+USER=opendkim
 EOS
   end
 end
