@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -569,6 +569,7 @@ int
 smfi_main (void)
 {
     gboolean success;
+    GError *error = NULL;
 
     libmilter_compatible_initialize();
 
@@ -576,7 +577,11 @@ smfi_main (void)
         g_object_unref(client);
     client = milter_client_new();
     setup_milter_client(client);
-    success = milter_client_main(client);
+    success = milter_client_run(client, &error);
+    if (!success) {
+        milter_error("failed to run main loop: %s", error->message);
+        g_error_free(error);
+    }
     g_object_unref(client);
     client = NULL;
 
@@ -806,16 +811,22 @@ int
 smfi_chgheader (SMFICTX *context, char *name, int index, char *value)
 {
     SmfiContextPrivate *priv;
+    GError *error = NULL;
 
     priv = SMFI_CONTEXT_GET_PRIVATE(context);
     if (!priv->client_context)
         return MI_FAILURE;
 
     if (milter_client_context_change_header(priv->client_context,
-                                            name, index, value))
+                                            name, index, value, &error)) {
         return MI_SUCCESS;
-    else
+    } else {
+        if (error) {
+            milter_error("failed to change header: %s", error->message);
+            g_error_free(error);
+        }
         return MI_FAILURE;
+    }
 }
 
 int
@@ -844,60 +855,88 @@ int
 smfi_chgfrom (SMFICTX *context, char *mail, char *arguments)
 {
     SmfiContextPrivate *priv;
+    GError *error = NULL;
 
     priv = SMFI_CONTEXT_GET_PRIVATE(context);
     if (!priv->client_context)
         return MI_FAILURE;
 
-    if (milter_client_context_change_from(priv->client_context, mail, arguments))
+    if (milter_client_context_change_from(priv->client_context, mail, arguments,
+                                          &error)) {
         return MI_SUCCESS;
-    else
+    } else {
+        if (error) {
+            milter_error("failed to change from: %s", error->message);
+            g_error_free(error);
+        }
         return MI_FAILURE;
+    }
 }
 
 int
 smfi_addrcpt (SMFICTX *context, char *recipient)
 {
     SmfiContextPrivate *priv;
+    GError *error = NULL;
 
     priv = SMFI_CONTEXT_GET_PRIVATE(context);
     if (!priv->client_context)
         return MI_FAILURE;
 
-    if (milter_client_context_add_recipient(priv->client_context, recipient, NULL))
+    if (milter_client_context_add_recipient(priv->client_context,
+                                            recipient, NULL, &error)) {
         return MI_SUCCESS;
-    else
+    } else {
+        if (error) {
+            milter_error("failed to add recipient: %s", error->message);
+            g_error_free(error);
+        }
         return MI_FAILURE;
+    }
 }
 
 int
 smfi_addrcpt_par (SMFICTX *context, char *recipient, char *args)
 {
     SmfiContextPrivate *priv;
+    GError *error = NULL;
 
     priv = SMFI_CONTEXT_GET_PRIVATE(context);
     if (!priv->client_context)
         return MI_FAILURE;
 
-    if (milter_client_context_add_recipient(priv->client_context, recipient, args))
+    if (milter_client_context_add_recipient(priv->client_context,
+                                            recipient, args, &error)) {
         return MI_SUCCESS;
-    else
+    } else {
+        if (error) {
+            milter_error("failed to add recipient: %s", error->message);
+            g_error_free(error);
+        }
         return MI_FAILURE;
+    }
 }
 
 int
 smfi_delrcpt (SMFICTX *context, char *recipient)
 {
     SmfiContextPrivate *priv;
+    GError *error = NULL;
 
     priv = SMFI_CONTEXT_GET_PRIVATE(context);
     if (!priv->client_context)
         return MI_FAILURE;
 
-    if (milter_client_context_delete_recipient(priv->client_context, recipient))
+    if (milter_client_context_delete_recipient(priv->client_context, recipient,
+                                               &error)) {
         return MI_SUCCESS;
-    else
+    } else {
+        if (error) {
+            milter_error("failed to delete recipient: %s", error->message);
+            g_error_free(error);
+        }
         return MI_FAILURE;
+    }
 }
 
 int
@@ -919,16 +958,23 @@ int
 smfi_replacebody (SMFICTX *context, unsigned char *new_body, int new_body_size)
 {
     SmfiContextPrivate *priv;
+    GError *error = NULL;
 
     priv = SMFI_CONTEXT_GET_PRIVATE(context);
     if (!priv->client_context)
         return MI_FAILURE;
 
     if (milter_client_context_replace_body(priv->client_context,
-                                           (char *)new_body, new_body_size))
+                                           (char *)new_body, new_body_size,
+                                           &error)) {
         return MI_SUCCESS;
-    else
+    } else {
+        if (error) {
+            milter_error("failed to replace body: %s", error->message);
+            g_error_free(error);
+        }
         return MI_FAILURE;
+    }
 }
 
 int
