@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -227,6 +227,8 @@ G_BEGIN_DECLS
  * @MILTER_CLIENT_CONTEXT_ERROR_INVALID_ACTION: Indicates
  * unexpected operation is requested on the context's
  * %MilterActionFlags.
+ * @MILTER_CLIENT_CONTEXT_ERROR_NULL: Indicates unexpected
+ * empty value is passed.
  *
  * These identify the variable errors that can occur while
  * calling %MilterClientContext functions.
@@ -237,7 +239,8 @@ typedef enum
     MILTER_CLIENT_CONTEXT_ERROR_IO_ERROR,
     MILTER_CLIENT_CONTEXT_ERROR_NULL,
     MILTER_CLIENT_CONTEXT_ERROR_INVALID_STATE,
-    MILTER_CLIENT_CONTEXT_ERROR_INVALID_ACTION
+    MILTER_CLIENT_CONTEXT_ERROR_INVALID_ACTION,
+    MILTER_CLIENT_CONTEXT_ERROR_EMPTY
 } MilterClientContextError;
 
 /**
@@ -561,6 +564,7 @@ gboolean             milter_client_context_insert_header     (MilterClientContex
  *         @name. (1-based) FIXME: should change 0-based?
  * @value: the header value. Use %NULL to delete the target
  *         header.
+ * @error: return location for an error, or %NULL.
  *
  * Changes a header that is located at @index in headers
  * that all of them are named @name. If @value is %NULL, the
@@ -578,7 +582,8 @@ gboolean             milter_client_context_insert_header     (MilterClientContex
 gboolean             milter_client_context_change_header     (MilterClientContext *context,
                                                               const gchar *name,
                                                               guint32      index,
-                                                              const gchar *value);
+                                                              const gchar *value,
+                                                              GError     **error);
 
 /**
  * milter_client_context_delete_header:
@@ -586,6 +591,7 @@ gboolean             milter_client_context_change_header     (MilterClientContex
  * @name: the header name.
  * @index: the index of headers that all of them are named
  * @name. (1-based) FIXME: should change 0-based?
+ * @error: return location for an error, or %NULL.
  *
  * Deletes a header that is located at @index in headers
  * that all of them are named @name. This function can be
@@ -600,7 +606,8 @@ gboolean             milter_client_context_change_header     (MilterClientContex
  */
 gboolean             milter_client_context_delete_header     (MilterClientContext *context,
                                                               const gchar *name,
-                                                              guint32      index);
+                                                              guint32      index,
+                                                              GError     **error);
 
 /**
  * milter_client_context_change_from:
@@ -608,6 +615,7 @@ gboolean             milter_client_context_delete_header     (MilterClientContex
  * @from: the new envelope from address.
  * @parameters: the ESMTP's 'MAIL FROM' parameter. It can be
  * %NULL.
+ * @error: return location for an error, or %NULL.
  *
  * Changes the envelope from address of the current message.
  * ESMTP's 'MAIL FROM' parameter can be set by
@@ -623,7 +631,8 @@ gboolean             milter_client_context_delete_header     (MilterClientContex
  */
 gboolean             milter_client_context_change_from       (MilterClientContext *context,
                                                               const gchar *from,
-                                                              const gchar *parameters);
+                                                              const gchar *parameters,
+                                                              GError     **error);
 
 /**
  * milter_client_context_add_recipient:
@@ -631,6 +640,7 @@ gboolean             milter_client_context_change_from       (MilterClientContex
  * @recipient: the new envelope recipient address.
  * @parameters: the ESMTP's 'RCPT TO' parameter. It can be
  * %NULL.
+ * @error: return location for an error, or %NULL.
  *
  * Adds a new envelope recipient address to the current
  * message.  ESMTP's 'RCPT TO' parameter can be set by
@@ -649,12 +659,14 @@ gboolean             milter_client_context_change_from       (MilterClientContex
  */
 gboolean             milter_client_context_add_recipient     (MilterClientContext *context,
                                                               const gchar *recipient,
-                                                              const gchar *parameters);
+                                                              const gchar *parameters,
+                                                              GError     **error);
 
 /**
  * milter_client_context_delete_recipient:
  * @context: a %MilterClientContext.
  * @recipient: the envelope recipient address to be removed.
+ * @error: return location for an error, or %NULL.
  *
  * Removes a envelope recipient that named @recipient. This
  * function can be called in
@@ -668,7 +680,8 @@ gboolean             milter_client_context_add_recipient     (MilterClientContex
  * Returns: %TRUE on success.
  */
 gboolean             milter_client_context_delete_recipient  (MilterClientContext *context,
-                                                              const gchar *recipient);
+                                                              const gchar *recipient,
+                                                              GError     **error);
 
 
 /**
@@ -676,6 +689,7 @@ gboolean             milter_client_context_delete_recipient  (MilterClientContex
  * @context: a %MilterClientContext.
  * @body: the new body.
  * @body_size: the size of @body.
+ * @error: return location for an error, or %NULL.
  *
  * Replaces the body of the current message with @body. This
  * function can be called in
@@ -690,7 +704,8 @@ gboolean             milter_client_context_delete_recipient  (MilterClientContex
  */
 gboolean             milter_client_context_replace_body      (MilterClientContext *context,
                                                               const gchar *body,
-                                                              gsize        body_size);
+                                                              gsize        body_size,
+                                                              GError     **error);
 
 /**
  * milter_client_context_progress:
@@ -889,6 +904,31 @@ void                 milter_client_context_reset_message_related_data
 guint                milter_client_context_get_n_processing_sessions
                                                        (MilterClientContext  *context);
 
+/**
+ * milter_client_context_set_packet_buffer_size:
+ * @context: a %MilterClientContext.
+ * @size: a packet buffer size in bytes. (The deafult is 0
+ *        bytes. It means buffering is disabled.)
+ *
+ * Sets the packet buffer size for the context. Packets on
+ * end-of-message are buffered until the buffer size is
+ * full. Packet buffering is for performance.
+ */
+void                 milter_client_context_set_packet_buffer_size
+                                                       (MilterClientContext *context,
+                                                        guint                size);
+
+/**
+ * milter_client_context_get_packet_buffer_size:
+ * @context: a %MilterClientContext.
+ *
+ * Gets the packet buffer size for the context.
+ *
+ * Returns: the packet buffer size for the context.
+ */
+guint                milter_client_context_get_packet_buffer_size
+                                                       (MilterClientContext *context);
+
 G_END_DECLS
 
 #endif /* __MILTER_CLIENT_CONTEXT_H__ */
