@@ -2,10 +2,50 @@
 /* Generated data (by glib-mkenums) */
 
 #include "milter-client-enum-types.h"
+#include "milter-client.h"
 #include "milter-client-context.h"
+#include "milter-client-runner.h"
+#include "milter-client-single-thread-runner.h"
 #include "milter-client-objects.h"
 
-/* enumerations from "milter-client-context.h" */
+/* enumerations from "milter-client.h" */
+GType
+milter_client_event_loop_backend_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { MILTER_CLIENT_EVENT_LOOP_BACKEND_GLIB, "MILTER_CLIENT_EVENT_LOOP_BACKEND_GLIB", "glib" },
+      { MILTER_CLIENT_EVENT_LOOP_BACKEND_LIBEV, "MILTER_CLIENT_EVENT_LOOP_BACKEND_LIBEV", "libev" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("MilterClientEventLoopBackend", values);
+  }
+  return etype;
+}
+GType
+milter_client_error_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { MILTER_CLIENT_ERROR_RUNNING, "MILTER_CLIENT_ERROR_RUNNING", "running" },
+      { MILTER_CLIENT_ERROR_UNIX_SOCKET, "MILTER_CLIENT_ERROR_UNIX_SOCKET", "unix-socket" },
+      { MILTER_CLIENT_ERROR_IO_ERROR, "MILTER_CLIENT_ERROR_IO_ERROR", "io-error" },
+      { MILTER_CLIENT_ERROR_THREAD, "MILTER_CLIENT_ERROR_THREAD", "thread" },
+      { MILTER_CLIENT_ERROR_PASSWORD_ENTRY, "MILTER_CLIENT_ERROR_PASSWORD_ENTRY", "password-entry" },
+      { MILTER_CLIENT_ERROR_GROUP_ENTRY, "MILTER_CLIENT_ERROR_GROUP_ENTRY", "group-entry" },
+      { MILTER_CLIENT_ERROR_DROP_PRIVILEGE, "MILTER_CLIENT_ERROR_DROP_PRIVILEGE", "drop-privilege" },
+      { MILTER_CLIENT_ERROR_DETACH_IO, "MILTER_CLIENT_ERROR_DETACH_IO", "detach-io" },
+      { MILTER_CLIENT_ERROR_DAEMONIZE, "MILTER_CLIENT_ERROR_DAEMONIZE", "daemonize" },
+      { MILTER_CLIENT_ERROR_NOT_LISTENED_YET, "MILTER_CLIENT_ERROR_NOT_LISTENED_YET", "not-listened-yet" },
+      { MILTER_CLIENT_ERROR_PROCESS, "MILTER_CLIENT_ERROR_PROCESS", "process" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("MilterClientError", values);
+  }
+  return etype;
+}
 GType
 milter_client_context_error_get_type (void)
 {
@@ -17,6 +57,7 @@ milter_client_context_error_get_type (void)
       { MILTER_CLIENT_CONTEXT_ERROR_NULL, "MILTER_CLIENT_CONTEXT_ERROR_NULL", "null" },
       { MILTER_CLIENT_CONTEXT_ERROR_INVALID_STATE, "MILTER_CLIENT_CONTEXT_ERROR_INVALID_STATE", "invalid-state" },
       { MILTER_CLIENT_CONTEXT_ERROR_INVALID_ACTION, "MILTER_CLIENT_CONTEXT_ERROR_INVALID_ACTION", "invalid-action" },
+      { MILTER_CLIENT_CONTEXT_ERROR_EMPTY, "MILTER_CLIENT_CONTEXT_ERROR_EMPTY", "empty" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("MilterClientContextError", values);
@@ -64,6 +105,32 @@ milter_client_context_state_get_type (void)
   }
   return etype;
 }
+GType
+milter_client_runner_error_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { MILTER_CLIENT_RUNNER_ERROR_RUNNING, "MILTER_CLIENT_RUNNER_ERROR_RUNNING", "running" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("MilterClientRunnerError", values);
+  }
+  return etype;
+}
+GType
+milter_client_single_thread_runner_error_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { MILTER_CLIENT_SINGLE_THREAD_RUNNER_ERROR_TODO, "MILTER_CLIENT_SINGLE_THREAD_RUNNER_ERROR_TODO", "todo" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("MilterClientSingleThreadRunnerError", values);
+  }
+  return etype;
+}
 
 /* Generated data ends here */
 
