@@ -1871,10 +1871,18 @@ milter_client_run (MilterClient *client, GError **error)
         const gchar *use_accept_loop_env;
         gboolean use_accept_loop = FALSE;
 
-        use_accept_loop_env = g_getenv("MILTER_USE_ACCEPT_LOOP");
-        if (use_accept_loop_env && strcmp(use_accept_loop_env, "yes") == 0)
+        if (priv->event_loop_backend == MILTER_CLIENT_EVENT_LOOP_BACKEND_LIBEV)
             use_accept_loop = TRUE;
 
+        use_accept_loop_env = g_getenv("MILTER_USE_ACCEPT_LOOP");
+        if (use_accept_loop_env) {
+            if (strcmp(use_accept_loop_env, "yes") == 0) {
+                use_accept_loop = TRUE;
+            } else {
+                use_accept_loop = FALSE;
+            }
+        }
+
         if (use_accept_loop) {
             if (!priv->accept_loop)
                 priv->accept_loop = milter_client_create_event_loop(client,
