@@ -808,10 +808,55 @@ _milter_marshal_VOID__OBJECT_OBJECT_ENUM (GClosure     *closure,
             data2);
 }
 
-/* VOID:ENUM (./milter-marshalers.list:18) */
+/* VOID:OBJECT,OBJECT,OBJECT (./milter-marshalers.list:18) */
+extern void _milter_marshal_VOID__OBJECT_OBJECT_OBJECT (GClosure     *closure,
+                                                        GValue       *return_value,
+                                                        guint         n_param_values,
+                                                        const GValue *param_values,
+                                                        gpointer      invocation_hint,
+                                                        gpointer      marshal_data);
+void
+_milter_marshal_VOID__OBJECT_OBJECT_OBJECT (GClosure     *closure,
+                                            GValue       *return_value G_GNUC_UNUSED,
+                                            guint         n_param_values,
+                                            const GValue *param_values,
+                                            gpointer      invocation_hint G_GNUC_UNUSED,
+                                            gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__OBJECT_OBJECT_OBJECT) (gpointer     data1,
+                                                           gpointer     arg_1,
+                                                           gpointer     arg_2,
+                                                           gpointer     arg_3,
+                                                           gpointer     data2);
+  register GMarshalFunc_VOID__OBJECT_OBJECT_OBJECT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__OBJECT_OBJECT_OBJECT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_object (param_values + 1),
+            g_marshal_value_peek_object (param_values + 2),
+            g_marshal_value_peek_object (param_values + 3),
+            data2);
+}
+
+/* VOID:ENUM (./milter-marshalers.list:19) */
 #define _milter_marshal_VOID__ENUM	g_cclosure_marshal_VOID__ENUM
 
-/* BOOLEAN:VOID (./milter-marshalers.list:19) */
+/* BOOLEAN:VOID (./milter-marshalers.list:20) */
 extern void _milter_marshal_BOOLEAN__VOID (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -854,7 +899,7 @@ _milter_marshal_BOOLEAN__VOID (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:POINTER (./milter-marshalers.list:20) */
+/* BOOLEAN:POINTER (./milter-marshalers.list:21) */
 extern void _milter_marshal_BOOLEAN__POINTER (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -899,7 +944,7 @@ _milter_marshal_BOOLEAN__POINTER (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:STRING (./milter-marshalers.list:21) */
+/* BOOLEAN:STRING (./milter-marshalers.list:22) */
 extern void _milter_marshal_BOOLEAN__STRING (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -944,7 +989,7 @@ _milter_marshal_BOOLEAN__STRING (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:STRING,STRING (./milter-marshalers.list:22) */
+/* BOOLEAN:STRING,STRING (./milter-marshalers.list:23) */
 extern void _milter_marshal_BOOLEAN__STRING_STRING (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
@@ -991,7 +1036,7 @@ _milter_marshal_BOOLEAN__STRING_STRING (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:STRING,UINT (./milter-marshalers.list:23) */
+/* BOOLEAN:STRING,UINT (./milter-marshalers.list:24) */
 extern void _milter_marshal_BOOLEAN__STRING_UINT (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -1038,7 +1083,7 @@ _milter_marshal_BOOLEAN__STRING_UINT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:STRING,UINT64 (./milter-marshalers.list:24) */
+/* BOOLEAN:STRING,UINT64 (./milter-marshalers.list:25) */
 extern void _milter_marshal_BOOLEAN__STRING_UINT64 (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
@@ -1085,7 +1130,7 @@ _milter_marshal_BOOLEAN__STRING_UINT64 (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* BOOLEAN:STRING,POINTER,UINT (./milter-marshalers.list:25) */
+/* BOOLEAN:STRING,POINTER,UINT (./milter-marshalers.list:26) */
 extern void _milter_marshal_BOOLEAN__STRING_POINTER_UINT (GClosure     *closure,
                                                           GValue       *return_value,
                                                           guint         n_param_values,
@@ -1134,7 +1179,7 @@ _milter_marshal_BOOLEAN__STRING_POINTER_UINT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* ENUM:STRING,POINTER,UINT (./milter-marshalers.list:26) */
+/* ENUM:STRING,POINTER,UINT (./milter-marshalers.list:27) */
 extern void _milter_marshal_ENUM__STRING_POINTER_UINT (GClosure     *closure,
                                                        GValue       *return_value,
                                                        guint         n_param_values,
@@ -1183,7 +1228,7 @@ _milter_marshal_ENUM__STRING_POINTER_UINT (GClosure     *closure,
   g_value_set_enum (return_value, v_return);
 }
 
-/* ENUM:OBJECT,OBJECT (./milter-marshalers.list:27) */
+/* ENUM:OBJECT,OBJECT (./milter-marshalers.list:28) */
 extern void _milter_marshal_ENUM__OBJECT_OBJECT (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -1230,7 +1275,7 @@ _milter_marshal_ENUM__OBJECT_OBJECT (GClosure     *closure,
   g_value_set_enum (return_value, v_return);
 }
 
-/* ENUM:STRING (./milter-marshalers.list:28) */
+/* ENUM:STRING (./milter-marshalers.list:29) */
 extern void _milter_marshal_ENUM__STRING (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -1275,7 +1320,7 @@ _milter_marshal_ENUM__STRING (GClosure     *closure,
   g_value_set_enum (return_value, v_return);
 }
 
-/* ENUM:STRING,STRING (./milter-marshalers.list:29) */
+/* ENUM:STRING,STRING (./milter-marshalers.list:30) */
 extern void _milter_marshal_ENUM__STRING_STRING (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -1322,7 +1367,7 @@ _milter_marshal_ENUM__STRING_STRING (GClosure     *closure,
   g_value_set_enum (return_value, v_return);
 }
 
-/* ENUM:VOID (./milter-marshalers.list:30) */
+/* ENUM:VOID (./milter-marshalers.list:31) */
 extern void _milter_marshal_ENUM__VOID (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
@@ -1365,7 +1410,7 @@ _milter_marshal_ENUM__VOID (GClosure     *closure,
   g_value_set_enum (return_value, v_return);
 }
 
-/* ENUM:ENUM (./milter-marshalers.list:31) */
+/* ENUM:ENUM (./milter-marshalers.list:32) */
 extern void _milter_marshal_ENUM__ENUM (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
@@ -1410,7 +1455,7 @@ _milter_marshal_ENUM__ENUM (GClosure     *closure,
   g_value_set_enum (return_value, v_return);
 }
 
-/* ENUM:STRING,UINT (./milter-marshalers.list:32) */
+/* ENUM:STRING,UINT (./milter-marshalers.list:33) */
 extern void _milter_marshal_ENUM__STRING_UINT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -1457,7 +1502,7 @@ _milter_marshal_ENUM__STRING_UINT (GClosure     *closure,
   g_value_set_enum (return_value, v_return);
 }
 
-/* ENUM:STRING,UINT64 (./milter-marshalers.list:33) */
+/* ENUM:STRING,UINT64 (./milter-marshalers.list:34) */
 extern void _milter_marshal_ENUM__STRING_UINT64 (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
