@@ -111,10 +111,6 @@ void             milter_utils_set_error_with_sub_error
                                               const gchar *format,
                                               ...);
 
-guint            milter_utils_timeout_add    (gdouble interval,
-                                              GSourceFunc function,
-                                              gpointer data);
-
 guint            milter_utils_flags_from_string
                                              (GType        flags_type,
                                               const gchar *flags_string);
@@ -151,6 +147,13 @@ gboolean         milter_utils_parse_file_mode(const gchar  *string,
                                               guint        *mode,
                                               gchar       **error_message);
 
+typedef enum {
+    MILTER_UTILS_READ_PIPE,
+    MILTER_UTILS_WRITE_PIPE
+} MilterUtilsPipeMode;
+
+
+
 G_END_DECLS
 
 #endif /* __MILTER_UTILS_H__ */
