@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -33,6 +33,8 @@
 #include "milter-manager-children.h"
 #include <milter/core/milter-marshalers.h>
 
+#define DEFAULT_FALLBACK_STATUS MILTER_STATUS_ACCEPT
+#define DEFAULT_FALLBACK_STATUS_AT_DISCONNECT MILTER_STATUS_TEMPORARY_FAILURE
 #define DEFAULT_MAINTENANCE_INTERVAL 10
 #define DEFAULT_CONNECTION_CHECK_INTERVAL 0
 
@@ -51,6 +53,7 @@ struct _MilterManagerConfigurationPrivate
     gchar *controller_connection_spec;
     gchar *manager_connection_spec;
     MilterStatus fallback_status;
+    MilterStatus fallback_status_at_disconnect;
     gchar *package_platform;
     gchar *package_options;
     gchar *effective_user;
@@ -72,6 +75,9 @@ struct _MilterManagerConfigurationPrivate
     gchar *custom_configuration_directory;
     GHashTable *locations;
     guint connection_check_interval;
+    MilterClientEventLoopBackend event_loop_backend;
+    guint n_workers;
+    guint default_packet_buffer_size;
 };
 
 enum
@@ -81,6 +87,7 @@ enum
     PROP_CONTROLLER_CONNECTION_SPEC,
     PROP_MANAGER_CONNECTION_SPEC,
     PROP_FALLBACK_STATUS,
+    PROP_FALLBACK_STATUS_AT_DISCONNECT,
     PROP_PACKAGE_PLATFORM,
     PROP_PACKAGE_OPTIONS,
     PROP_EFFECTIVE_USER,
@@ -100,14 +107,17 @@ enum
     PROP_MAX_CONNECTIONS,
     PROP_MAX_FILE_DESCRIPTORS,
     PROP_CUSTOM_CONFIGURATION_DIRECTORY,
-    PROP_CONNECTION_CHECK_INTERVAL
+    PROP_CONNECTION_CHECK_INTERVAL,
+    PROP_EVENT_LOOP_BACKEND,
+    PROP_N_WORKER_PROCESSES,
+    PROP_DEFAULT_PACKET_BUFFER_SIZE
 };
 
 enum
 {
     CONNECTED,
     TO_XML,
-    LAST_SIGNAL,
+    LAST_SIGNAL
 };
 
 static gint signals[LAST_SIGNAL] = {0};
@@ -170,10 +180,20 @@ milter_manager_configuration_class_init (MilterManagerConfigurationClass *klass)
                              "Fallback status",
                              "The fallback status of the milter-manager",
                              MILTER_TYPE_STATUS,
-                             MILTER_STATUS_ACCEPT,
+                             DEFAULT_FALLBACK_STATUS,
                              G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
     g_object_class_install_property(gobject_class, PROP_FALLBACK_STATUS, spec);
 
+    spec = g_param_spec_enum("fallback-status-at-disconnect",
+                             "Fallback status at disconnect",
+                             "The fallback status at disconnect "
+                             "of the milter-manager",
+                             MILTER_TYPE_STATUS,
+                             DEFAULT_FALLBACK_STATUS_AT_DISCONNECT,
+                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+    g_object_class_install_property(gobject_class,
+                                    PROP_FALLBACK_STATUS_AT_DISCONNECT, spec);
+
     spec = g_param_spec_string("package-platform",
                                "Package platform",
                                "The package platform of the milter-manager",
@@ -371,6 +391,33 @@ milter_manager_configuration_class_init (MilterManagerConfigurationClass *klass)
                                     spec);
 
 
+    spec = g_param_spec_enum("event-loop-backend",
+                             "Event loop backend",
+                             "The event loop backend of the configuration",
+                             MILTER_TYPE_CLIENT_EVENT_LOOP_BACKEND,
+                             MILTER_CLIENT_EVENT_LOOP_BACKEND_GLIB,
+                             G_PARAM_READWRITE);
+    g_object_class_install_property(gobject_class,
+                                    PROP_EVENT_LOOP_BACKEND,
+                                    spec);
+
+    spec = g_param_spec_uint("n-workers",
+                             "Number of worker processes",
+                             "The number of worker processes of the client",
+                             0, MILTER_CLIENT_MAX_N_WORKERS, 0,
+                             G_PARAM_READWRITE);
+    g_object_class_install_property(gobject_class, PROP_N_WORKER_PROCESSES, spec);
+
+    spec = g_param_spec_uint("default-packet-buffer-size",
+                             "Default packet buffer size",
+                             "The default packet buffer size of client contexts "
+                             "of the client",
+                             0, G_MAXUINT, 0,
+                             G_PARAM_READWRITE);
+    g_object_class_install_property(gobject_class,
+                                    PROP_DEFAULT_PACKET_BUFFER_SIZE, spec);
+
+
     signals[CONNECTED] =
         g_signal_new("connected",
                      G_TYPE_FROM_CLASS(klass),
@@ -420,6 +467,8 @@ milter_manager_configuration_init (MilterManagerConfiguration *configuration)
                                             g_free,
                                             (GDestroyNotify)g_dataset_destroy);
     priv->connection_check_interval = DEFAULT_CONNECTION_CHECK_INTERVAL;
+    priv->n_workers = 0;
+    priv->default_packet_buffer_size = 0;
 
     config_dir_env = g_getenv("MILTER_MANAGER_CONFIG_DIR");
     if (config_dir_env)
@@ -482,6 +531,10 @@ set_property (GObject      *object,
         milter_manager_configuration_set_fallback_status(
             config, g_value_get_enum(value));
         break;
+    case PROP_FALLBACK_STATUS_AT_DISCONNECT:
+        milter_manager_configuration_set_fallback_status_at_disconnect(
+            config, g_value_get_enum(value));
+        break;
     case PROP_PACKAGE_PLATFORM:
         milter_manager_configuration_set_package_platform(
             config, g_value_get_string(value));
@@ -562,6 +615,18 @@ set_property (GObject      *object,
         milter_manager_configuration_set_connection_check_interval(
             config, g_value_get_uint(value));
         break;
+    case PROP_EVENT_LOOP_BACKEND:
+        milter_manager_configuration_set_event_loop_backend(
+            config, g_value_get_enum(value));
+        break;
+    case PROP_N_WORKER_PROCESSES:
+        milter_manager_configuration_set_n_workers(config, g_value_get_uint(value));
+        break;
+    case PROP_DEFAULT_PACKET_BUFFER_SIZE:
+        milter_manager_configuration_set_default_packet_buffer_size(
+            config,
+            g_value_get_uint(value));
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -590,6 +655,9 @@ get_property (GObject    *object,
     case PROP_FALLBACK_STATUS:
         g_value_set_enum(value, priv->fallback_status);
         break;
+    case PROP_FALLBACK_STATUS_AT_DISCONNECT:
+        g_value_set_enum(value, priv->fallback_status_at_disconnect);
+        break;
     case PROP_PACKAGE_PLATFORM:
         g_value_set_string(value, priv->package_platform);
         break;
@@ -650,6 +718,15 @@ get_property (GObject    *object,
     case PROP_CONNECTION_CHECK_INTERVAL:
         g_value_set_uint(value, priv->connection_check_interval);
         break;
+    case PROP_EVENT_LOOP_BACKEND:
+        g_value_set_enum(value, priv->event_loop_backend);
+        break;
+    case PROP_N_WORKER_PROCESSES:
+        g_value_set_uint(value, priv->n_workers);
+        break;
+    case PROP_DEFAULT_PACKET_BUFFER_SIZE:
+        g_value_set_uint(value, priv->default_packet_buffer_size);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -1529,7 +1606,10 @@ MilterStatus
 milter_manager_configuration_get_fallback_status
                                      (MilterManagerConfiguration *configuration)
 {
-    return MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration)->fallback_status;
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    return priv->fallback_status;
 }
 
 void
@@ -1543,6 +1623,27 @@ milter_manager_configuration_set_fallback_status
     priv->fallback_status = status;
 }
 
+MilterStatus
+milter_manager_configuration_get_fallback_status_at_disconnect
+                                     (MilterManagerConfiguration *configuration)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    return priv->fallback_status_at_disconnect;
+}
+
+void
+milter_manager_configuration_set_fallback_status_at_disconnect
+                                     (MilterManagerConfiguration *configuration,
+                                      MilterStatus status)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    priv->fallback_status_at_disconnect = status;
+}
+
 const gchar *
 milter_manager_configuration_get_package_platform
                                      (MilterManagerConfiguration *configuration)
@@ -1702,13 +1803,16 @@ clear_manager (MilterManagerConfigurationPrivate *priv)
 
     priv->privilege_mode = FALSE;
     priv->daemon = FALSE;
-    priv->fallback_status = MILTER_STATUS_ACCEPT;
+    priv->fallback_status = DEFAULT_FALLBACK_STATUS;
+    priv->fallback_status_at_disconnect = DEFAULT_FALLBACK_STATUS_AT_DISCONNECT;
     priv->maintenance_interval = DEFAULT_MAINTENANCE_INTERVAL;
     priv->suspend_time_on_unacceptable =
         MILTER_CLIENT_DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE;
     priv->max_connections = 0;
     priv->max_file_descriptors = 0;
     priv->connection_check_interval = DEFAULT_CONNECTION_CHECK_INTERVAL;
+    priv->n_workers = 0;
+    priv->default_packet_buffer_size = 0;
 }
 
 static void
@@ -1857,7 +1961,8 @@ milter_manager_configuration_to_xml_string (MilterManagerConfiguration *configur
         if (g_stat(full_path, &status) == 0) {
             gchar *content;
 
-            content = g_strdup_printf("%" G_GSSIZE_FORMAT, status.st_mtime);
+            content = g_strdup_printf("%" G_GINT64_FORMAT,
+                                      (gint64)status.st_mtime);
             milter_utils_xml_append_text_element(string,
                                                  "last-modified", content,
                                                  indent + 2);
@@ -2001,6 +2106,62 @@ milter_manager_configuration_set_connection_check_interval (MilterManagerConfigu
     priv->connection_check_interval = interval_in_seconds;
 }
 
+MilterClientEventLoopBackend
+milter_manager_configuration_get_event_loop_backend (MilterManagerConfiguration *configuration)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    return priv->event_loop_backend;
+}
+
+void
+milter_manager_configuration_set_event_loop_backend (MilterManagerConfiguration *configuration,
+                                                     MilterClientEventLoopBackend backend)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    priv->event_loop_backend = backend;
+}
+
+guint
+milter_manager_configuration_get_n_workers (MilterManagerConfiguration *configuration)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    return priv->n_workers;
+}
+
+void
+milter_manager_configuration_set_n_workers (MilterManagerConfiguration *configuration,
+                                            guint                       n_workers)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    priv->n_workers = n_workers;
+}
+
+guint
+milter_manager_configuration_get_default_packet_buffer_size (MilterManagerConfiguration *configuration)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    return priv->default_packet_buffer_size;
+}
+
+void
+milter_manager_configuration_set_default_packet_buffer_size (MilterManagerConfiguration *configuration,
+                                                             guint                       size)
+{
+    MilterManagerConfigurationPrivate *priv;
+
+    priv = MILTER_MANAGER_CONFIGURATION_GET_PRIVATE(configuration);
+    priv->default_packet_buffer_size = size;
+}
 
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
