@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -189,7 +189,7 @@ milter_manager_leader_check_connection (MilterManagerLeader *leader)
     }
 
     if (priv->children &&
-        milter_manager_children_is_waiting_reply(priv->children)) {
+        !milter_manager_children_is_waiting_reply(priv->children)) {
         milter_debug("[%u] [leader][connection-check][%s][skip] "
                      "not milter turn",
                      priv->tag, state_nick);
@@ -220,7 +220,7 @@ milter_manager_leader_check_connection (MilterManagerLeader *leader)
         milter_statistics("[session][disconnected][%g](%u)", elapsed, priv->tag);
 
         fallback_status =
-            milter_manager_configuration_get_fallback_status(
+            milter_manager_configuration_get_fallback_status_at_disconnect(
                 priv->configuration);
         if (milter_need_debug_log()) {
             gchar *fallback_status_nick;
@@ -668,7 +668,7 @@ cb_insert_header (MilterReplySignals *_reply,
                      priv->tag, error->message);
         milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader), error);
         g_error_free(error);
-    } else {
+    } else if (value) {
         milter_statistics(
             "[session][header][add](%u): <%s>=<%s>", priv->tag, name, value);
     }
@@ -681,12 +681,18 @@ cb_change_header (MilterReplySignals *_reply,
 {
     MilterManagerLeader *leader = user_data;
     MilterManagerLeaderPrivate *priv;
+    GError *error = NULL;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     milter_client_context_change_header(priv->client_context,
-                                        name, index, value);
+                                        name, index, value, &error);
 
-    if (value) {
+    if (error) {
+        milter_error("[%u] [leader][error][insert-header] %s",
+                     priv->tag, error->message);
+        milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader), error);
+        g_error_free(error);
+    } else if (value) {
         milter_statistics(
             "[session][header][add](%u): <%s>=<%s>", priv->tag, name, value);
     }
@@ -699,9 +705,19 @@ cb_delete_header (MilterReplySignals *_reply,
 {
     MilterManagerLeader *leader = user_data;
     MilterManagerLeaderPrivate *priv;
+    GError *error = NULL;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
-    milter_client_context_delete_header(priv->client_context, name, index);
+    if (milter_client_context_delete_header(priv->client_context, name, index,
+                                            &error)) {
+        milter_debug(
+            "[%u] [leader][header][delete] <%s>[%u]", priv->tag, name, index);
+    } else {
+        milter_error("[%u] [leader][error][delete-header] %s",
+                     priv->tag, error->message);
+        milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader), error);
+        g_error_free(error);
+    }
 }
 
 static void
@@ -711,9 +727,20 @@ cb_change_from (MilterReplySignals *_reply,
 {
     MilterManagerLeader *leader = user_data;
     MilterManagerLeaderPrivate *priv;
+    GError *error = NULL;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
-    milter_client_context_change_from(priv->client_context, from, parameters);
+    if (milter_client_context_change_from(priv->client_context, from, parameters,
+                                          &error)) {
+        milter_debug(
+            "[%u] [leader][from][change] <%s>(%s)",
+            priv->tag, from, parameters ? parameters : "NULL");
+    } else {
+        milter_error("[%u] [leader][error][change-from] %s",
+                     priv->tag, error->message);
+        milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader), error);
+        g_error_free(error);
+    }
 }
 
 static void
@@ -723,10 +750,20 @@ cb_add_recipient (MilterReplySignals *_reply,
 {
     MilterManagerLeader *leader = user_data;
     MilterManagerLeaderPrivate *priv;
+    GError *error = NULL;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
-    milter_client_context_add_recipient(priv->client_context,
-                                        recipient, parameters);
+    if (milter_client_context_add_recipient(priv->client_context,
+                                            recipient, parameters,
+                                            &error)) {
+        milter_debug("[%u] [leader][recipient][add] <%s>(%s)",
+                     priv->tag, recipient, parameters ? parameters : "NULL");
+    } else {
+        milter_error("[%u] [leader][error][add-recipient] %s",
+                     priv->tag, error->message);
+        milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader), error);
+        g_error_free(error);
+    }
 }
 
 static void
@@ -736,9 +773,19 @@ cb_delete_recipient (MilterReplySignals *_reply,
 {
     MilterManagerLeader *leader = user_data;
     MilterManagerLeaderPrivate *priv;
+    GError *error = NULL;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
-    milter_client_context_delete_recipient(priv->client_context, recipient);
+    if (milter_client_context_delete_recipient(priv->client_context, recipient,
+                                               &error)) {
+        milter_debug("[%u] [leader][recipient][delete] <%s>",
+                     priv->tag, recipient);
+    } else {
+        milter_error("[%u] [leader][error][delete-recipient] %s",
+                     priv->tag, error->message);
+        milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader), error);
+        g_error_free(error);
+    }
 }
 
 static void
@@ -748,9 +795,21 @@ cb_replace_body (MilterReplySignals *_reply,
 {
     MilterManagerLeader *leader = user_data;
     MilterManagerLeaderPrivate *priv;
+    GError *error = NULL;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
-    milter_client_context_replace_body(priv->client_context, chunk, chunk_size);
+    if (milter_client_context_replace_body(priv->client_context,
+                                           chunk, chunk_size, &error)) {
+        milter_debug("[%u] [leader][body][replace] <%.10s%s>(%zd)>",
+                     priv->tag,
+                     chunk, chunk_size > 10 ? "..." : "",
+                     chunk_size);
+    } else {
+        milter_error("[%u] [leader][error][replace-body] %s",
+                     priv->tag, error->message);
+        milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader), error);
+        g_error_free(error);
+    }
 }
 
 static void
@@ -933,11 +992,14 @@ milter_manager_leader_negotiate (MilterManagerLeader *leader,
 {
     MilterManagerLeaderPrivate *priv;
     MilterStatus fallback_status;
+    MilterEventLoop *event_loop;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_NEGOTIATE;
 
-    priv->children = milter_manager_children_new(priv->configuration);
+    event_loop = milter_agent_get_event_loop(MILTER_AGENT(priv->client_context));
+    priv->children = milter_manager_children_new(priv->configuration,
+                                                 event_loop);
     milter_manager_configuration_setup_children(priv->configuration,
                                                 priv->children,
                                                 priv->client_context);
