@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -104,6 +104,18 @@ static void   sessions_finished           (MilterClient *client,
                                            guint         n_finished_sessions);
 static void   cb_leader_finished          (MilterFinishedEmittable *emittable,
                                            gpointer user_data);
+static guint  get_n_workers               (MilterClient *client);
+static void   set_n_workers               (MilterClient *client,
+                                           guint         n_workers);
+static MilterClientEventLoopBackend get_event_loop_backend
+                                          (MilterClient *client);
+static void   set_event_loop_backend      (MilterClient *client,
+                                           MilterClientEventLoopBackend backend);
+static guint  get_default_packet_buffer_size
+                                          (MilterClient *client);
+static void   set_default_packet_buffer_size
+                                          (MilterClient *client,
+                                           guint         size);
 
 static void
 milter_manager_class_init (MilterManagerClass *klass)
@@ -136,6 +148,14 @@ milter_manager_class_init (MilterManagerClass *klass)
     client_class->set_effective_group = set_effective_group;
     client_class->get_maintenance_interval = get_maintenance_interval;
     client_class->set_maintenance_interval = set_maintenance_interval;
+    client_class->get_n_workers = get_n_workers;
+    client_class->set_n_workers = set_n_workers;
+    client_class->get_event_loop_backend = get_event_loop_backend;
+    client_class->set_event_loop_backend = set_event_loop_backend;
+    client_class->get_default_packet_buffer_size =
+        get_default_packet_buffer_size;
+    client_class->set_default_packet_buffer_size =
+        set_default_packet_buffer_size;
     client_class->maintain = maintain;
     client_class->sessions_finished = sessions_finished;
 
@@ -180,14 +200,19 @@ configuration_set_manager (MilterManagerConfiguration *configuration,
 }
 
 static void
-dispose_periodical_connection_checker (MilterManagerPrivate *priv)
+dispose_periodical_connection_checker (MilterManager *manager)
 {
+    MilterManagerPrivate *priv;
+    MilterEventLoop *loop;
+
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
     priv->current_periodical_connection_check_interval = 0;
 
     if (priv->periodical_connection_checker_id == 0)
         return;
 
-    g_source_remove(priv->periodical_connection_checker_id);
+    loop = milter_client_get_process_loop(MILTER_CLIENT(manager));
+    milter_event_loop_remove(loop, priv->periodical_connection_checker_id);
     priv->periodical_connection_checker_id = 0;
 }
 
@@ -214,11 +239,13 @@ dispose_finished_leaders (MilterManagerPrivate *priv)
 static void
 dispose (GObject *object)
 {
+    MilterManager *manager;
     MilterManagerPrivate *priv;
 
-    priv = MILTER_MANAGER_GET_PRIVATE(object);
+    manager = MILTER_MANAGER(object);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
 
-    dispose_periodical_connection_checker(priv);
+    dispose_periodical_connection_checker(manager);
     dispose_finished_leaders(priv);
 
     if (priv->configuration) {
@@ -345,18 +372,21 @@ start_periodical_connection_checker (MilterManager *manager)
     if (interval > 0) {
         if (priv->periodical_connection_checker_id == 0 ||
             interval != priv->current_periodical_connection_check_interval) {
+            MilterEventLoop *loop;
             milter_debug("[manager][connection-check][start] <%u> -> <%u>: %s",
                          priv->current_periodical_connection_check_interval,
                          interval,
                          priv->periodical_connection_checker_id == 0 ?
                          "initial" : "update");
-            dispose_periodical_connection_checker(priv);
+            dispose_periodical_connection_checker(manager);
             priv->current_periodical_connection_check_interval = interval;
+            loop = milter_client_get_process_loop(MILTER_CLIENT(manager));
             priv->periodical_connection_checker_id =
-                g_timeout_add(interval * 1000, connection_check, manager);
+                milter_event_loop_add_timeout(loop, interval,
+                                              connection_check, manager);
         }
     } else {
-        dispose_periodical_connection_checker(priv);
+        dispose_periodical_connection_checker(manager);
     }
 }
 
@@ -689,7 +719,7 @@ cb_leader_finished (MilterFinishedEmittable *emittable, gpointer user_data)
         }
         if (!priv->leaders) {
             milter_debug("[manager][connection-check][dispose] no leaders");
-            dispose_periodical_connection_checker(priv);
+            dispose_periodical_connection_checker(finish_data->manager);
         }
     }
 
@@ -986,6 +1016,33 @@ set_maintenance_interval (MilterClient *client, guint n_sessions)
                                                           n_sessions);
 }
 
+static guint
+get_default_packet_buffer_size (MilterClient *client)
+{
+    MilterManager *manager;
+    MilterManagerPrivate *priv;
+    MilterManagerConfiguration *configuration;
+
+    manager = MILTER_MANAGER(client);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    configuration = priv->configuration;
+    return milter_manager_configuration_get_default_packet_buffer_size(configuration);
+}
+
+static void
+set_default_packet_buffer_size (MilterClient *client, guint size)
+{
+    MilterManager *manager;
+    MilterManagerPrivate *priv;
+    MilterManagerConfiguration *configuration;
+
+    manager = MILTER_MANAGER(client);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    configuration = priv->configuration;
+    milter_manager_configuration_set_default_packet_buffer_size(configuration,
+                                                                size);
+}
+
 static void
 maintain (MilterClient *client)
 {
@@ -1008,6 +1065,44 @@ sessions_finished (MilterClient *client, guint n_finished_sessions)
     dispose_finished_leaders(priv);
 }
 
+static guint
+get_n_workers (MilterClient *client)
+{
+    MilterManagerPrivate *priv;
+
+    priv = MILTER_MANAGER_GET_PRIVATE(client);
+    return milter_manager_configuration_get_n_workers(priv->configuration);
+}
+
+static void
+set_n_workers (MilterClient *client, guint n_workers)
+{
+    MilterManagerPrivate *priv;
+
+    priv = MILTER_MANAGER_GET_PRIVATE(client);
+    milter_manager_configuration_set_n_workers(priv->configuration, n_workers);
+}
+
+static MilterClientEventLoopBackend
+get_event_loop_backend (MilterClient *client)
+{
+    MilterManagerPrivate *priv;
+
+    priv = MILTER_MANAGER_GET_PRIVATE(client);
+    return milter_manager_configuration_get_event_loop_backend(priv->configuration);
+}
+
+static void
+set_event_loop_backend (MilterClient *client,
+                        MilterClientEventLoopBackend backend)
+{
+    MilterManagerPrivate *priv;
+
+    priv = MILTER_MANAGER_GET_PRIVATE(client);
+    milter_manager_configuration_set_event_loop_backend(priv->configuration,
+                                                        backend);
+}
+
 MilterManagerConfiguration *
 milter_manager_get_configuration (MilterManager *manager)
 {
