@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2009-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2009-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -32,6 +32,7 @@
 void test_tag (void);
 void test_socket_address (void);
 void test_n_processing_sessions (void);
+void test_packet_buffer_size (void);
 
 static MilterClientContext *context;
 
@@ -104,9 +105,14 @@ test_tag (void)
     cut_assert_equal_uint(29, milter_reader_get_tag(reader));
 
     milter_agent_set_reader(agent, NULL);
-    cut_assert_equal_uint(0, milter_reader_get_tag(reader));
+    cut_assert_equal_uint(29, milter_reader_get_tag(reader));
     milter_agent_set_writer(agent, NULL);
-    cut_assert_equal_uint(0, milter_writer_get_tag(writer));
+    cut_assert_equal_uint(29, milter_writer_get_tag(writer));
+
+    milter_reader_set_tag(reader, 2929);
+    cut_assert_equal_uint(2929, milter_reader_get_tag(reader));
+    milter_writer_set_tag(writer, 2929);
+    cut_assert_equal_uint(2929, milter_writer_get_tag(writer));
 
     milter_agent_set_reader(agent, reader);
     cut_assert_equal_uint(29, milter_reader_get_tag(reader));
@@ -138,6 +144,16 @@ test_n_processing_sessions (void)
         0, milter_client_context_get_n_processing_sessions(context));
 }
 
+void
+test_packet_buffer_size (void)
+{
+    cut_assert_equal_uint(0,
+                          milter_client_context_get_packet_buffer_size(context));
+    milter_client_context_set_packet_buffer_size(context, 4096);
+    cut_assert_equal_uint(4096,
+                          milter_client_context_get_packet_buffer_size(context));
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
