@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -25,6 +25,7 @@
 
 #include <gcutter.h>
 #include "milter-test-compatible.h"
+#include "milter-test-glib-compatible.h"
 #include "milter-assertions.h"
 #include "milter-test-client.h"
 #include "milter-test-server.h"
@@ -37,9 +38,13 @@ typedef enum
 } MilterTestErrno;
 
 const gchar   *milter_test_get_base_dir    (void);
+const gchar   *milter_test_get_build_dir   (void);
+const gchar   *milter_test_get_source_dir  (void);
+gchar         *milter_test_get_tmp_dir     (void);
 
 void           milter_test_write_data_to_io_channel
-                                           (GIOChannel  *io_channel,
+                                           (MilterEventLoop *loop,
+                                            GIOChannel  *io_channel,
                                             const gchar *data,
                                             gsize        data_size);
 gboolean       milter_test_read_data_from_io_channel
@@ -50,7 +55,8 @@ gboolean       milter_test_decode_io_channel
                                            (MilterDecoder  *decoder,
                                             GIOChannel     *channel);
 guint          milter_test_io_add_decode_watch
-                                           (GIOChannel     *channel,
+                                           (MilterEventLoop *loop,
+                                            GIOChannel     *channel,
                                             MilterDecoder  *decoder);
 
 gboolean       milter_test_equal_symbols_table
@@ -58,7 +64,9 @@ gboolean       milter_test_equal_symbols_table
                                             GHashTable *table2);
 gchar         *milter_test_inspect_symbols_table
                                            (GHashTable *table);
-void           milter_test_pump_all_events (void);
+void           milter_test_pump_all_events (MilterEventLoop *loop);
+
+MilterEventLoop *milter_test_event_loop_new(void);
 
 G_END_DECLS
 
