@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2009-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2009-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -38,8 +38,8 @@ void test_setmlreply (void);
 void test_addheader (void);
 void test_chgheader (void);
 void test_insheader (void);
-void data_cfgfrom (void);
-void test_cfgfrom (gconstpointer data);
+void data_chgfrom (void);
+void test_chgfrom (gconstpointer data);
 void test_addrcpt (void);
 void test_addrcpt_par (void);
 void data_delrcpt (void);
@@ -48,6 +48,10 @@ void test_progress (void);
 void test_quarantine (void);
 void test_replacebody (void);
 
+static MilterEventLoop *loop;
+
+static GError *actual_error;
+
 static SmfiContext *context;
 static MilterClientContext *client_context;
 static MilterCommandEncoder *command_encoder;
@@ -57,9 +61,6 @@ static GIOChannel *channel;
 static MilterWriter *writer;
 static GString *expected_output;
 
-static gchar *packet;
-static gsize packet_size;
-
 static gboolean send_progress;
 
 static gchar *macro_name;
@@ -249,13 +250,32 @@ static struct smfiDesc smfilter = {
     xxfi_negotiate,
 };
 
+static void
+cb_error (MilterErrorEmittable *emittable, GError *error)
+{
+    actual_error = g_error_copy(error);
+}
+
+static void
+setup_signals (MilterClientContext *context)
+{
+    g_signal_connect(context, "error", G_CALLBACK(cb_error), NULL);
+}
 
 void
-setup (void)
+cut_setup (void)
 {
+    GError *error = NULL;
+
     smfi_register(smfilter);
 
+    loop = milter_test_event_loop_new();
+
+    actual_error = NULL;
+
     client_context = milter_client_context_new(NULL);
+    setup_signals(client_context);
+    milter_agent_set_event_loop(MILTER_AGENT(client_context), loop);
     expected_output = NULL;
 
     context = smfi_context_new(client_context);
@@ -264,14 +284,12 @@ setup (void)
     g_io_channel_set_encoding(channel, NULL, NULL);
     writer = milter_writer_io_channel_new(channel);
     milter_agent_set_writer(MILTER_AGENT(client_context), writer);
-    milter_agent_start(MILTER_AGENT(client_context), NULL);
+    milter_agent_start(MILTER_AGENT(client_context), &error);
+    gcut_assert_error(error);
 
     command_encoder = MILTER_COMMAND_ENCODER(milter_command_encoder_new());
     reply_encoder = MILTER_REPLY_ENCODER(milter_reply_encoder_new());
 
-    packet = NULL;
-    packet_size = 0;
-
     send_progress = FALSE;
 
     add_header = FALSE;
@@ -311,17 +329,8 @@ setup (void)
     set_mlreply_result = MI_FAILURE;
 }
 
-static void
-packet_free (void)
-{
-    if (packet)
-        g_free(packet);
-    packet = NULL;
-    packet_size = 0;
-}
-
 void
-teardown (void)
+cut_teardown (void)
 {
     if (context)
         g_object_unref(context);
@@ -332,6 +341,10 @@ teardown (void)
         g_io_channel_unref(channel);
     if (writer)
         g_object_unref(writer);
+    if (loop)
+        g_object_unref(loop);
+    if (actual_error)
+        g_error_free(actual_error);
     if (expected_output)
         g_string_free(expected_output, TRUE);
 
@@ -340,8 +353,6 @@ teardown (void)
     if (reply_encoder)
         g_object_unref(reply_encoder);
 
-    packet_free();
-
     if (macro_name)
         g_free(macro_name);
     if (macro_value)
@@ -372,12 +383,22 @@ teardown (void)
         g_free(reply_message);
 }
 
+static void
+pump_all_events (void)
+{
+    milter_test_pump_all_events(loop);
+    gcut_assert_error(actual_error);
+}
+
 static GError *
-feed (void)
+feed (const gchar *packet, gsize packet_size)
 {
     GError *error = NULL;
 
     milter_client_context_feed(client_context, packet, packet_size, &error);
+    if (!error) {
+        pump_all_events();
+    }
 
     return error;
 }
@@ -388,6 +409,8 @@ feed_negotiate (void)
     MilterOption *option;
     MilterActionFlags action;
     MilterStepFlags step;
+    const gchar *packet;
+    gsize packet_size;
     GError *error = NULL;
 
     action = gcut_flags_get_all(MILTER_TYPE_ACTION_FLAGS, &error);
@@ -399,9 +422,7 @@ feed_negotiate (void)
                                             &packet, &packet_size,
                                             option);
     g_object_unref(option);
-    gcut_assert_error(feed());
-
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 static void
@@ -409,6 +430,8 @@ feed_connect (void)
 {
     struct sockaddr *address;
     socklen_t address_size;
+    const gchar *packet;
+    gsize packet_size;
     GError *error = NULL;
 
     milter_connection_parse_spec("inet:2929@192.168.1.5",
@@ -419,86 +442,90 @@ feed_connect (void)
                                           "mx.example.com",
                                           address, address_size);
     g_free(address);
-    gcut_assert_error(feed());
-
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 static void
 feed_helo (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_command_encoder_encode_helo(command_encoder,
                                        &packet, &packet_size,
                                        "mx.example.com");
-    gcut_assert_error(feed());
-
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 static void
 feed_envelope_from (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_command_encoder_encode_envelope_from(command_encoder,
                                                 &packet, &packet_size,
                                                 "sender@example.com");
-    gcut_assert_error(feed());
-
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 static void
 feed_envelope_recipient (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_command_encoder_encode_envelope_recipient(command_encoder,
                                                      &packet, &packet_size,
                                                      "receiver@example.com");
-    gcut_assert_error(feed());
-
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 static void
 feed_data (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_command_encoder_encode_data(command_encoder,
                                        &packet, &packet_size);
-    gcut_assert_error(feed());
-
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 static void
 feed_header (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_command_encoder_encode_header(command_encoder,
                                          &packet, &packet_size,
                                          "From", "sender@example.com");
-    gcut_assert_error(feed());
-
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 static void
 feed_end_of_header (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     milter_command_encoder_encode_end_of_header(command_encoder,
                                                 &packet, &packet_size);
-    gcut_assert_error(feed());
-
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 static void
 feed_body (void)
 {
     const gchar body[] = "Hello\n\nBye\n";
+    const gchar *packet;
+    gsize packet_size;
 
     milter_command_encoder_encode_body(command_encoder,
                                        &packet, &packet_size,
                                        body, strlen(body), NULL);
-    gcut_assert_error(feed());
-
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 }
 
 static void
@@ -531,6 +558,8 @@ void
 test_getsymval (void)
 {
     GHashTable *macros;
+    const gchar *packet;
+    gsize packet_size;
 
     macros = gcut_hash_table_string_string_new("{mail_addr}", "kou@example.com",
                                                NULL);
@@ -539,15 +568,14 @@ test_getsymval (void)
                                                MILTER_COMMAND_ENVELOPE_FROM,
                                                macros);
     g_hash_table_unref(macros);
-    gcut_assert_error(feed());
-    packet_free();
+    gcut_assert_error(feed(packet, packet_size));
 
     cut_assert_equal_string(NULL, macro_value);
     macro_name = g_strdup("mail_addr");
     milter_command_encoder_encode_envelope_from(command_encoder,
                                                 &packet, &packet_size,
                                                 "<kou@example.com>");
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_string("kou@example.com", macro_value);
 }
 
@@ -583,6 +611,9 @@ data_setreply (void)
 void
 test_setreply (gconstpointer data)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     reply_return_code = g_strdup(gcut_data_get_string(data, "return-code"));
     reply_extended_code = g_strdup(gcut_data_get_string(data, "extended-code"));
     reply_message = g_strdup(gcut_data_get_string(data, "message"));
@@ -590,7 +621,7 @@ test_setreply (gconstpointer data)
 
     milter_command_encoder_encode_data(command_encoder,
                                        &packet, &packet_size);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(gcut_data_get_int(data, "expected-return-status"),
                          set_reply_result);
 }
@@ -598,10 +629,13 @@ test_setreply (gconstpointer data)
 void
 test_setmlreply (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     set_mlreply = TRUE;
     milter_command_encoder_encode_end_of_header(command_encoder,
                                                 &packet, &packet_size);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(MI_SUCCESS, set_mlreply_result);
     cut_assert_equal_string_with_free(
         "450 4.2.0 Rejected by Greylist\r\n"
@@ -612,6 +646,8 @@ test_setmlreply (void)
 void
 test_addheader (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     GString *actual_data;
 
     move_to_body_state();
@@ -623,18 +659,16 @@ test_addheader (void)
     milter_command_encoder_encode_end_of_message(command_encoder,
                                                  &packet, &packet_size,
                                                  NULL, 0);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
 
 
     expected_output = g_string_new(NULL);
 
-    packet_free();
     milter_reply_encoder_encode_add_header(reply_encoder,
                                            &packet, &packet_size,
                                            header_name, header_value);
     g_string_append_len(expected_output, packet, packet_size);
 
-    packet_free();
     milter_reply_encoder_encode_continue(reply_encoder, &packet, &packet_size);
     g_string_append_len(expected_output, packet, packet_size);
 
@@ -646,6 +680,8 @@ test_addheader (void)
 void
 test_insheader (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     GString *actual_data;
 
     move_to_body_state();
@@ -658,19 +694,17 @@ test_insheader (void)
     milter_command_encoder_encode_end_of_message(command_encoder,
                                                  &packet, &packet_size,
                                                  NULL, 0);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
 
 
     expected_output = g_string_new(NULL);
 
-    packet_free();
     milter_reply_encoder_encode_insert_header(reply_encoder,
                                               &packet, &packet_size,
                                               header_index,
                                               header_name, header_value);
     g_string_append_len(expected_output, packet, packet_size);
 
-    packet_free();
     milter_reply_encoder_encode_continue(reply_encoder, &packet, &packet_size);
     g_string_append_len(expected_output, packet, packet_size);
 
@@ -682,6 +716,8 @@ test_insheader (void)
 void
 test_chgheader (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     GString *actual_data;
 
     move_to_body_state();
@@ -694,12 +730,11 @@ test_chgheader (void)
     milter_command_encoder_encode_end_of_message(command_encoder,
                                                  &packet, &packet_size,
                                                  NULL, 0);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
 
 
     expected_output = g_string_new(NULL);
 
-    packet_free();
     milter_reply_encoder_encode_change_header(reply_encoder,
                                               &packet, &packet_size,
                                               header_name,
@@ -707,7 +742,6 @@ test_chgheader (void)
                                               header_value);
     g_string_append_len(expected_output, packet, packet_size);
 
-    packet_free();
     milter_reply_encoder_encode_continue(reply_encoder, &packet, &packet_size);
     g_string_append_len(expected_output, packet, packet_size);
 
@@ -717,7 +751,7 @@ test_chgheader (void)
 }
 
 void
-data_cfgfrom (void)
+data_chgfrom (void)
 {
     cut_add_data("from",
                  g_strsplit("kou@localhost", " ", 2),
@@ -740,8 +774,10 @@ data_cfgfrom (void)
 }
 
 void
-test_cfgfrom (gconstpointer data)
+test_chgfrom (gconstpointer data)
 {
+    const gchar *packet;
+    gsize packet_size;
     GString *actual_data;
     gchar * const *test_data = data;
 
@@ -760,12 +796,11 @@ test_cfgfrom (gconstpointer data)
     milter_command_encoder_encode_end_of_message(command_encoder,
                                                  &packet, &packet_size,
                                                  NULL, 0);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
 
     expected_output = g_string_new(NULL);
 
     if (change_from && change_from[0] != '\0') {
-        packet_free();
         milter_reply_encoder_encode_change_from(reply_encoder,
                                                 &packet, &packet_size,
                                                 change_from,
@@ -773,7 +808,6 @@ test_cfgfrom (gconstpointer data)
         g_string_append_len(expected_output, packet, packet_size);
     }
 
-    packet_free();
     milter_reply_encoder_encode_continue(reply_encoder, &packet, &packet_size);
     g_string_append_len(expected_output, packet, packet_size);
 
@@ -785,6 +819,8 @@ test_cfgfrom (gconstpointer data)
 void
 test_addrcpt (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     GString *actual_data;
 
     move_to_body_state();
@@ -795,18 +831,16 @@ test_addrcpt (void)
     milter_command_encoder_encode_end_of_message(command_encoder,
                                                  &packet, &packet_size,
                                                  NULL, 0);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
 
 
     expected_output = g_string_new(NULL);
 
-    packet_free();
     milter_reply_encoder_encode_add_recipient(reply_encoder,
                                               &packet, &packet_size,
                                               add_recipient, NULL);
     g_string_append_len(expected_output, packet, packet_size);
 
-    packet_free();
     milter_reply_encoder_encode_continue(reply_encoder, &packet, &packet_size);
     g_string_append_len(expected_output, packet, packet_size);
 
@@ -818,6 +852,8 @@ test_addrcpt (void)
 void
 test_addrcpt_par (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     GString *actual_data;
 
     move_to_body_state();
@@ -829,19 +865,17 @@ test_addrcpt_par (void)
     milter_command_encoder_encode_end_of_message(command_encoder,
                                                  &packet, &packet_size,
                                                  NULL, 0);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
 
 
     expected_output = g_string_new(NULL);
 
-    packet_free();
     milter_reply_encoder_encode_add_recipient(reply_encoder,
                                               &packet, &packet_size,
                                               add_recipient,
                                               add_recipient_parameters);
     g_string_append_len(expected_output, packet, packet_size);
 
-    packet_free();
     milter_reply_encoder_encode_continue(reply_encoder, &packet, &packet_size);
     g_string_append_len(expected_output, packet, packet_size);
 
@@ -867,6 +901,8 @@ data_delrcpt (void)
 void
 test_delrcpt (gconstpointer data)
 {
+    const gchar *packet;
+    gsize packet_size;
     GString *actual_data;
     const gchar *test_data = data;
 
@@ -877,19 +913,17 @@ test_delrcpt (gconstpointer data)
     milter_command_encoder_encode_end_of_message(command_encoder,
                                                  &packet, &packet_size,
                                                  NULL, 0);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
 
     expected_output = g_string_new(NULL);
 
     if (delete_recipient && delete_recipient[0] != '\0') {
-        packet_free();
         milter_reply_encoder_encode_delete_recipient(reply_encoder,
                                                      &packet, &packet_size,
                                                      delete_recipient);
         g_string_append_len(expected_output, packet, packet_size);
     }
 
-    packet_free();
     milter_reply_encoder_encode_continue(reply_encoder, &packet, &packet_size);
     g_string_append_len(expected_output, packet, packet_size);
 
@@ -901,22 +935,22 @@ test_delrcpt (gconstpointer data)
 void
 test_progress (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     GString *actual_data;
     const gchar fqdn[] = "delian";
 
     send_progress = TRUE;
     milter_command_encoder_encode_helo(command_encoder,
                                        &packet, &packet_size, fqdn);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
 
 
     expected_output = g_string_new(NULL);
 
-    packet_free();
     milter_reply_encoder_encode_progress(reply_encoder, &packet, &packet_size);
     g_string_append_len(expected_output, packet, packet_size);
 
-    packet_free();
     milter_reply_encoder_encode_continue(reply_encoder, &packet, &packet_size);
     g_string_append_len(expected_output, packet, packet_size);
 
@@ -928,6 +962,8 @@ test_progress (void)
 void
 test_quarantine (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     GString *actual_data;
 
     move_to_body_state();
@@ -936,17 +972,15 @@ test_quarantine (void)
     milter_command_encoder_encode_end_of_message(command_encoder,
                                                  &packet, &packet_size,
                                                  NULL, 0);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
 
 
     expected_output = g_string_new(NULL);
 
-    packet_free();
     milter_reply_encoder_encode_quarantine(reply_encoder, &packet, &packet_size,
                                            quarantine_reason);
     g_string_append_len(expected_output, packet, packet_size);
 
-    packet_free();
     milter_reply_encoder_encode_continue(reply_encoder, &packet, &packet_size);
     g_string_append_len(expected_output, packet, packet_size);
 
@@ -958,6 +992,8 @@ test_quarantine (void)
 void
 test_replacebody (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     GString *actual_data;
     gsize packed_size;
 
@@ -968,19 +1004,17 @@ test_replacebody (void)
     milter_command_encoder_encode_end_of_message(command_encoder,
                                                  &packet, &packet_size,
                                                  NULL, 0);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
     cut_assert_equal_int(MI_SUCCESS, replace_body_result);
 
 
     expected_output = g_string_new(NULL);
 
-    packet_free();
     milter_reply_encoder_encode_replace_body(reply_encoder,
                                              &packet, &packet_size,
                                              body->str, body->len, &packed_size);
     g_string_append_len(expected_output, packet, packet_size);
 
-    packet_free();
     milter_reply_encoder_encode_continue(reply_encoder, &packet, &packet_size);
     g_string_append_len(expected_output, packet, packet_size);
 
