@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -58,8 +58,10 @@ void test_header (void);
 void test_header_with_protocol_version2 (void);
 void test_header_no_reply (void);
 void test_end_of_header (void);
+void test_end_of_header_with_protocol_version2 (void);
 void test_end_of_header_no_reply (void);
 void test_body (void);
+void test_body_with_protocol_version2 (void);
 void test_body_no_reply (void);
 void data_important_status (void);
 void test_important_status (gconstpointer data);
@@ -69,10 +71,13 @@ void test_reading_timeout (void);
 void test_connection_timeout (void);
 void test_end_of_message_timeout (void);
 void test_writing_timeout (void);
+void test_end_of_message_with_protocol_version2 (void);
 
 static gchar *scenario_dir;
 static MilterManagerTestScenario *main_scenario;
 
+static MilterEventLoop *loop;
+
 static MilterManagerConfiguration *config;
 static MilterManagerChildren *children;
 static MilterOption *option;
@@ -127,6 +132,11 @@ static struct sockaddr *actual_address;
 static MilterLogLevelFlags original_log_level;
 static gboolean default_handler_disconnected;
 
+#define collect_n_received(event)                                       \
+    milter_manager_test_clients_collect_n_received(                     \
+        test_clients,                                                   \
+        milter_manager_test_client_get_n_ ## event ## _received)
+
 void
 cut_startup (void)
 {
@@ -332,7 +342,7 @@ start_client (guint port, GArray *arguments)
     MilterManagerTestClient *client;
     GError *error = NULL;
 
-    client = milter_manager_test_client_new(port, NULL);
+    client = milter_manager_test_client_new(port, NULL, loop);
     milter_manager_test_client_set_arguments(client, arguments);
     test_clients = g_list_append(test_clients, client);
     if (!milter_manager_test_client_run(client, &error)) {
@@ -376,14 +386,17 @@ clear_n_emitted (void)
 void
 cut_setup (void)
 {
-    scenario_dir = g_build_filename(milter_test_get_base_dir(),
+    scenario_dir = g_build_filename(milter_test_get_source_dir(),
+                                    "test",
                                     "fixtures",
                                     "children",
                                     NULL);
     main_scenario = NULL;
 
+    loop = milter_test_event_loop_new();
+
     config = milter_manager_configuration_new(NULL);
-    children = milter_manager_children_new(config);
+    children = milter_manager_children_new(config, loop);
     setup_signals(children);
 
     launcher = NULL;
@@ -502,6 +515,9 @@ cut_teardown (void)
 
     if (actual_address)
         g_free(actual_address);
+
+    if (loop)
+        g_object_unref(loop);
 }
 
 static void
@@ -558,7 +574,7 @@ cb_timeout_waiting (gpointer data)
 #define wait_reply(expected, actual)            \
     cut_trace_with_info_expression(             \
         wait_reply_helper(expected, &actual),   \
-        wait_reply(actual))
+        wait_reply(expected, actual))
 
 static void
 wait_reply_helper (guint expected, guint *actual)
@@ -567,14 +583,17 @@ wait_reply_helper (guint expected, guint *actual)
     guint timeout_waiting_id;
 
     cut_assert_true(milter_manager_children_is_waiting_reply(children));
-    timeout_waiting_id = g_timeout_add(500, cb_timeout_waiting,
-                                       &timeout_waiting);
+    timeout_waiting_id = milter_event_loop_add_timeout(loop, 0.5,
+                                                       cb_timeout_waiting,
+                                                       &timeout_waiting);
     while (timeout_waiting && expected > *actual) {
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     }
-    g_source_remove(timeout_waiting_id);
+    milter_event_loop_remove(loop, timeout_waiting_id);
 
-    cut_assert_true(timeout_waiting);
+    cut_assert_true(timeout_waiting,
+                    cut_message("timeout: expect:<%u> actual:<%u>",
+                                expected, *actual));
     cut_assert_equal_uint(expected, *actual);
 }
 
@@ -624,12 +643,13 @@ wait_finished_helper (void)
     guint timeout_waiting_id;
     guint n_finished_emitted_before = n_finished_emitted;
 
-    timeout_waiting_id = g_timeout_add(500, cb_timeout_waiting,
-                                       &timeout_waiting);
+    timeout_waiting_id = milter_event_loop_add_timeout(loop, 0.5,
+                                                       cb_timeout_waiting,
+                                                       &timeout_waiting);
     while (timeout_waiting && n_finished_emitted == n_finished_emitted_before) {
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     }
-    g_source_remove(timeout_waiting_id);
+    milter_event_loop_remove(loop, timeout_waiting_id);
 
     cut_assert_true(timeout_waiting, cut_message("timeout"));
     cut_assert_operator_uint(n_finished_emitted, > , n_finished_emitted_before);
@@ -646,12 +666,13 @@ wait_children_error_helper (void)
     gboolean timeout_waiting = TRUE;
     guint timeout_waiting_id;
 
-    timeout_waiting_id = g_timeout_add(500, cb_timeout_waiting,
-                                       &timeout_waiting);
+    timeout_waiting_id = milter_event_loop_add_timeout(loop, 0.5,
+                                                       cb_timeout_waiting,
+                                                       &timeout_waiting);
     while (timeout_waiting && !actual_error) {
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     }
-    g_source_remove(timeout_waiting_id);
+    milter_event_loop_remove(loop, timeout_waiting_id);
 
     cut_assert_true(timeout_waiting, cut_message("timeout"));
     cut_assert_not_null(actual_error);
@@ -715,6 +736,7 @@ setup_process_launcher (MilterManagerChildren *children)
     MilterWriter *writer;
 
     launcher = milter_manager_process_launcher_new();
+    milter_agent_set_event_loop(MILTER_AGENT(launcher), loop);
 
     read_channel = create_io_channel();
     reader = milter_reader_io_channel_new(read_channel);
@@ -881,10 +903,12 @@ assert_response_common (MilterManagerTestScenario *scenario, const gchar *group)
         gboolean timeout_emitted = FALSE;
         guint timeout_id;
 
-        timeout_id = g_timeout_add(100, cb_timeout_emitted, &timeout_emitted);
+        timeout_id = milter_event_loop_add_timeout(loop, 0.1,
+                                                   cb_timeout_emitted,
+                                                   &timeout_emitted);
         while (!timeout_emitted && n_emitted > get_n_emitted(response))
-            g_main_context_iteration(NULL, TRUE);
-        g_source_remove(timeout_id);
+            milter_event_loop_iterate(loop, TRUE);
+        milter_event_loop_remove(loop, timeout_id);
         cut_assert_false(timeout_emitted);
     }
 
@@ -1091,7 +1115,8 @@ data_scenario (void)
                  g_strdup("envelope-recipient.txt"), g_free,
                  "data", g_strdup("data.txt"), g_free,
                  "header", g_strdup("header.txt"), g_free,
-                 "end-of-header", g_strdup("end-of-header.txt"), g_free);
+                 "end-of-header", g_strdup("end-of-header.txt"), g_free,
+                 NULL);
 }
 
 void
@@ -1106,7 +1131,7 @@ test_scenario (gconstpointer data)
                                                 scenario_name));
     cut_trace(setup_parameters(main_scenario, children));
 
-    cut_trace(milter_manager_test_scenario_start_clients(main_scenario));
+    cut_trace(milter_manager_test_scenario_start_clients(main_scenario, loop));
 
     for (node = milter_manager_test_scenario_get_eggs(main_scenario);
          node;
@@ -1483,8 +1508,10 @@ test_header_with_protocol_version2 (void)
 
     cut_trace(test_envelope_recipient());
 
+    cut_assert_equal_uint(0, collect_n_received(data));
     milter_manager_children_header(children, name, value);
-    wait_reply(6, n_continue_emitted);
+    wait_reply(5, n_continue_emitted);
+    cut_assert_equal_uint(1, collect_n_received(data));
 }
 
 void
@@ -1503,6 +1530,7 @@ test_header_no_reply (void)
     cut_trace(test_data());
 
     milter_manager_children_header(children, name, value);
+    milter_test_pump_all_events(loop);
     cut_assert_false(milter_manager_children_is_waiting_reply(children));
 }
 
@@ -1515,6 +1543,21 @@ test_end_of_header (void)
     wait_reply(7, n_continue_emitted);
 }
 
+void
+test_end_of_header_with_protocol_version2 (void)
+{
+    arguments_append(arguments1,
+                     "--negotiate-version", "2",
+                     NULL);
+
+    cut_trace(test_envelope_recipient());
+
+    cut_assert_equal_uint(0, collect_n_received(data));
+    milter_manager_children_end_of_header(children);
+    wait_reply(5, n_continue_emitted);
+    cut_assert_equal_uint(1, collect_n_received(data));
+}
+
 void
 test_end_of_header_no_reply (void)
 {
@@ -1528,6 +1571,7 @@ test_end_of_header_no_reply (void)
     cut_trace(test_header());
 
     milter_manager_children_end_of_header(children);
+    milter_test_pump_all_events(loop);
     cut_assert_false(milter_manager_children_is_waiting_reply(children));
 }
 
@@ -1542,6 +1586,23 @@ test_body (void)
     wait_reply(8, n_continue_emitted);
 }
 
+void
+test_body_with_protocol_version2 (void)
+{
+    const gchar chunk[] = "message body";
+
+    arguments_append(arguments1,
+                     "--negotiate-version", "2",
+                     NULL);
+
+    cut_trace(test_envelope_recipient());
+
+    cut_assert_equal_uint(0, collect_n_received(data));
+    milter_manager_children_body(children, chunk, strlen(chunk));
+    wait_reply(5, n_continue_emitted);
+    cut_assert_equal_uint(1, collect_n_received(data));
+}
+
 void
 test_body_no_reply (void)
 {
@@ -1555,8 +1616,10 @@ test_body_no_reply (void)
                      "--negotiate-flags", "no-reply-body",
                      NULL);
     cut_trace(test_end_of_header());
+    milter_test_pump_all_events(loop);
 
     milter_manager_children_body(children, chunk, strlen(chunk));
+    milter_test_pump_all_events(loop);
     cut_assert_false(milter_manager_children_is_waiting_reply(children));
 }
 
@@ -1647,7 +1710,8 @@ data_important_status (void)
                  label " - temporary failure - discard",                \
                  test_data(state, TEMPORARY_FAILURE, DISCARD), g_free,  \
                  label " - reject - discard",                           \
-                 test_data(state, REJECT, DISCARD), g_free)
+                 test_data(state, REJECT, DISCARD), g_free,             \
+                 NULL)
 
     add_data("connect", CONNECT);
     add_data("envelope from", ENVELOPE_FROM);
@@ -1671,7 +1735,8 @@ data_important_status (void)
                  "body - skip - discard",
                  test_data(BODY, SKIP, DISCARD), g_free,
                  "body - accept - skip",
-                 test_data(BODY, ACCEPT, SKIP), g_free);
+                 test_data(BODY, ACCEPT, SKIP), g_free,
+                 NULL);
     add_data("end of message", END_OF_MESSAGE);
 
 #undef add_data
@@ -1717,7 +1782,8 @@ data_not_important_status (void)
                  label " - discard - accept",                           \
                  test_data(state, DISCARD, ACCEPT), g_free,             \
                  label " - discard - reject",                           \
-                 test_data(state, DISCARD, REJECT), g_free);
+                 test_data(state, DISCARD, REJECT), g_free,             \
+                 NULL);
 
     add_data("connect", CONNECT);
     add_data("envelope from", ENVELOPE_FROM);
@@ -1737,7 +1803,8 @@ data_not_important_status (void)
                  "body - skip - default",
                  test_data(BODY, SKIP, DEFAULT), g_free,
                  "body - skip - not change",
-                 test_data(BODY, SKIP, NOT_CHANGE), g_free);
+                 test_data(BODY, SKIP, NOT_CHANGE), g_free,
+                 NULL);
     add_data("end of message", END_OF_MESSAGE);
 
 #undef add_data
@@ -1781,7 +1848,8 @@ cb_writing_timeout (MilterServerContext *context, gpointer user_data)
 {
     GIOChannel *channel = user_data;
 
-    gcut_string_io_channel_set_limit(channel, 0);
+    if (channel)
+        gcut_string_io_channel_set_limit(channel, 0);
     n_writing_timeout_emitted++;
 }
 
@@ -1843,7 +1911,7 @@ test_no_negotiation (void)
     child10027 = milter_manager_children_get_children(children)->next->data;
 
     cut_assert_true(milter_manager_children_negotiate(children, option, NULL));
-    g_main_context_iteration(NULL, FALSE);
+    milter_event_loop_iterate(loop, FALSE);
     cut_assert_equal_string(
         cut_take_printf("[%u] [children][error][connection] [%u] "
                         "Failed to connect to inet:10026@localhost: "
@@ -1912,6 +1980,7 @@ test_end_of_message_timeout (void)
 
     child = milter_manager_children_get_children(children)->data;
     prepare_timeout_test(child, NULL);
+    milter_server_context_set_writing_timeout(MILTER_SERVER_CONTEXT(child), 1);
 
     milter_manager_children_end_of_message(children, "end", strlen("end"));
 
@@ -1953,7 +2022,7 @@ test_writing_timeout (void)
     g_io_channel_unref(channel);
     child = milter_manager_children_get_children(children)->next->data;
     milter_agent_set_writer(MILTER_AGENT(child), writer);
-    milter_writer_start(writer, NULL);
+    milter_writer_start(writer, loop);
     g_object_unref(writer);
 
     prepare_timeout_test(child, channel);
@@ -2001,6 +2070,7 @@ test_reading_timeout (void)
 
     child = milter_manager_children_get_children(children)->next->data;
     prepare_timeout_test(child, NULL);
+    milter_server_context_set_writing_timeout(MILTER_SERVER_CONTEXT(child), 1);
 
     address.sin_family = AF_INET;
     address.sin_port = g_htons(50443);
@@ -2021,6 +2091,21 @@ test_reading_timeout (void)
         error_message->str);
 }
 
+void
+test_end_of_message_with_protocol_version2 (void)
+{
+    arguments_append(arguments1,
+                     "--negotiate-version", "2",
+                     NULL);
+
+    cut_trace(test_envelope_recipient());
+
+    cut_assert_equal_uint(0, collect_n_received(data));
+    milter_manager_children_end_of_message(children, NULL, 0);
+    wait_reply(5, n_continue_emitted);
+    cut_assert_equal_uint(1, collect_n_received(data));
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
