@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -36,6 +36,8 @@ void data_launch_error (void);
 void test_already_launched (void);
 void test_run (void);
 
+static MilterEventLoop *loop;
+
 static MilterManagerProcessLauncher *launcher;
 static MilterManagerLaunchCommandEncoder *command_encoder;
 static MilterManagerReplyEncoder *reply_encoder;
@@ -46,9 +48,6 @@ static GIOChannel *output_channel;
 static GString *expected_packet;
 static gchar *expected_error_message;
 
-static gchar *packet;
-static gsize packet_size;
-
 static gchar *current_locale;
 
 static guint idle_id;
@@ -84,7 +83,7 @@ setup_input_io (void)
     g_object_unref(input_reader);
 
     writer = milter_writer_io_channel_new(channel);
-    milter_writer_start(writer, NULL);
+    milter_writer_start(writer, loop);
 
     g_io_channel_unref(channel);
 }
@@ -106,17 +105,22 @@ setup_output_io (void)
 static void
 setup_io (void)
 {
+    GError *error = NULL;
+
     setup_input_io();
     setup_output_io();
-    milter_agent_start(MILTER_AGENT(launcher), NULL);
+    milter_agent_start(MILTER_AGENT(launcher), &error);
+    gcut_assert_error(error);
 }
 
 void
-setup (void)
+cut_setup (void)
 {
     MilterEncoder *encoder;
 
+    loop = milter_test_event_loop_new();
     launcher = milter_manager_process_launcher_new();
+    milter_agent_set_event_loop(MILTER_AGENT(launcher), loop);
 
     setup_io();
 
@@ -127,15 +131,12 @@ setup (void)
 
     expected_error_message = NULL;
 
-    packet = NULL;
-    packet_size = 0;
-
     idle_id = 0;
     timeout_id = 0;
 }
 
 void
-teardown (void)
+cut_teardown (void)
 {
     if (launcher)
         g_object_unref(launcher);
@@ -150,24 +151,36 @@ teardown (void)
     if (reply_encoder)
         g_object_unref(reply_encoder);
 
-    if (packet)
-        g_free(packet);
-
     if (expected_packet)
         g_string_free(expected_packet, TRUE);
     if (expected_error_message)
         g_free(expected_error_message);
 
     if (idle_id > 0)
-        g_source_remove(idle_id);
+        milter_event_loop_remove(loop, idle_id);
     if (timeout_id > 0)
-        g_source_remove(timeout_id);
+        milter_event_loop_remove(loop, timeout_id);
+
+    if (loop)
+        g_object_unref(loop);
+}
+
+static void
+write_packet (const gchar *packet, gsize packet_size)
+{
+    GError *error = NULL;
+
+    milter_writer_write(writer, packet, packet_size, &error);
+    gcut_assert_error(error);
+    milter_writer_flush(writer, &error);
+    gcut_assert_error(error);
 }
 
 void
 test_launch (void)
 {
-    GError *error = NULL;
+    const gchar *packet;
+    gsize packet_size;
     GString *output;
     const gchar command_line[] = "/bin/echo -n";
     const gchar *user_name;
@@ -176,11 +189,9 @@ test_launch (void)
     milter_manager_launch_command_encoder_encode_launch(command_encoder,
                                                         &packet, &packet_size,
                                                         command_line, user_name);
-    milter_writer_write(writer, packet, packet_size, NULL, &error);
-    gcut_assert_error(error);
-    milter_test_pump_all_events();
+    cut_trace(write_packet(packet, packet_size));
+    milter_test_pump_all_events(loop);
 
-    g_free(packet);
     milter_manager_reply_encoder_encode_success(reply_encoder,
                                                 &packet, &packet_size);
     output = gcut_string_io_channel_get_string(output_channel);
@@ -191,9 +202,10 @@ test_launch (void)
 void
 test_already_launched (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     GString *output;
     GString *expected_packet;
-    GError *error = NULL;
     const gchar command_line[] = "/bin/cat";
     const gchar *user_name;
 
@@ -201,16 +213,13 @@ test_already_launched (void)
     milter_manager_launch_command_encoder_encode_launch(command_encoder,
                                                         &packet, &packet_size,
                                                         command_line, user_name);
-    milter_writer_write(writer, packet, packet_size, NULL, &error);
-    milter_writer_write(writer, packet, packet_size, NULL, &error);
-    gcut_assert_error(error);
-    milter_test_pump_all_events();
+    cut_trace(write_packet(packet, packet_size));
+    cut_trace(write_packet(packet, packet_size));
+    milter_test_pump_all_events(loop);
 
-    g_free(packet);
     milter_manager_reply_encoder_encode_success(reply_encoder,
                                                 &packet, &packet_size);
     expected_packet = g_string_new_len(packet, packet_size);
-    g_free(packet);
     milter_manager_reply_encoder_encode_error(reply_encoder,
                                               &packet, &packet_size,
                                               "already launched: </bin/cat>");
@@ -263,8 +272,9 @@ data_launch_error (void)
 void
 test_launch_error (gconstpointer data)
 {
+    const gchar *packet;
+    gsize packet_size;
     GString *output;
-    GError *error = NULL;
     const gchar *command;
     const gchar *user_name;
     const gchar *expected_error_message;
@@ -277,11 +287,9 @@ test_launch_error (gconstpointer data)
     milter_manager_launch_command_encoder_encode_launch(command_encoder,
                                                         &packet, &packet_size,
                                                         command, user_name);
-    milter_writer_write(writer, packet, packet_size, NULL, &error);
-    gcut_assert_error(error);
-    milter_test_pump_all_events();
+    cut_trace(write_packet(packet, packet_size));
+    milter_test_pump_all_events(loop);
 
-    g_free(packet);
     milter_manager_reply_encoder_encode_error(reply_encoder,
                                               &packet, &packet_size,
                                               expected_error_message);
@@ -295,7 +303,7 @@ cb_idle_shutdown (gpointer data)
 {
     MilterManagerProcessLauncher *launcher = data;
 
-    milter_agent_shutdown(MILTER_AGENT(launcher));
+    milter_manager_process_launcher_shutdown(launcher);
 
     return FALSE;
 }
@@ -315,8 +323,10 @@ test_run (void)
 {
     gboolean timeout_emitted = FALSE;
 
-    idle_id = g_idle_add(cb_idle_shutdown, launcher);
-    timeout_id = g_timeout_add(100, cb_timeout_mark_emitted, &timeout_emitted);
+    idle_id = milter_event_loop_add_idle(loop, cb_idle_shutdown, launcher);
+    timeout_id = milter_event_loop_add_timeout(loop, 0.1,
+                                               cb_timeout_mark_emitted,
+                                               &timeout_emitted);
 
     milter_manager_process_launcher_run(launcher);
 
