@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,6 +40,10 @@ static gchar *group = NULL;
 static gchar *unix_socket_group = NULL;
 static guint unix_socket_mode = 0660;
 static MilterClient *client = NULL;
+static guint n_workers = 0;
+static MilterClientEventLoopBackend event_loop_backend =
+    MILTER_CLIENT_EVENT_LOOP_BACKEND_GLIB;
+static guint packet_buffer_size = 0;
 
 static gboolean
 print_version (const gchar *option_name,
@@ -99,6 +103,32 @@ parse_unix_socket_mode (const gchar *option_name,
     return success;
 }
 
+static gboolean
+parse_event_loop_backend (const gchar *option_name,
+                        const gchar *value,
+                        gpointer data,
+                        GError **error)
+{
+    GEnumClass *backend_enum;
+    GEnumValue *backend_value;
+
+    backend_enum = g_type_class_ref(MILTER_TYPE_CLIENT_EVENT_LOOP_BACKEND);
+    backend_value = g_enum_get_value_by_nick(backend_enum, value);
+    g_type_class_unref(backend_enum);
+
+    if (backend_value) {
+        event_loop_backend = backend_value->value;
+        return TRUE;
+    } else {
+        g_set_error(error,
+                    G_OPTION_ERROR,
+                    G_OPTION_ERROR_BAD_VALUE,
+                    _("invalid name: %s"),
+                    value);
+        return FALSE;
+    }
+}
+
 static const GOptionEntry option_entries[] =
 {
     {"connection-spec", 's', 0, G_OPTION_ARG_CALLBACK, parse_spec_arg,
@@ -125,6 +155,13 @@ static const GOptionEntry option_entries[] =
      N_("Change UNIX domain socket group to GROUP"), "GROUP"},
     {"unix-socket-mode", 0, 0, G_OPTION_ARG_CALLBACK, parse_unix_socket_mode,
      N_("Change UNIX domain socket mode to MODE (default: 0660)"), "MODE"},
+    {"n-workers", 0, 0, G_OPTION_ARG_INT, &n_workers,
+     N_("Run N_WORKERS processes (default: 0)"), "N_WORKERS"},
+    {"event-loop-backend", 0, 0, G_OPTION_ARG_CALLBACK, parse_event_loop_backend,
+     N_("Use BACKEND as event loop backend (default: glib)"), "[glib|libev]"},
+    {"packet-buffer-size", 0, 0, G_OPTION_ARG_INT, &packet_buffer_size,
+     N_("Use SIZE as packet buffer size in bytes. 0 disables packet buffering. "
+        "(default: 0; disabled)"), "SIZE"},
     {"version", 0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, print_version,
      N_("Show version"), NULL},
     {NULL}
@@ -496,6 +533,10 @@ main (int argc, char *argv[])
     milter_client_set_effective_group(client, group);
     milter_client_set_unix_socket_mode(client, unix_socket_mode);
     milter_client_set_unix_socket_group(client, unix_socket_group);
+    milter_client_set_event_loop_backend(client, event_loop_backend);
+    if (n_workers > 0)
+        milter_client_set_n_workers(client, n_workers);
+    milter_client_set_default_packet_buffer_size(client, packet_buffer_size);
     if (spec)
         success = milter_client_set_connection_spec(client, spec, &error);
     if (success)
@@ -509,7 +550,11 @@ main (int argc, char *argv[])
 
         setup_client_signals(client);
         sigint_handler = signal(SIGINT, cb_sigint_shutdown_client);
-        success = milter_client_main(client);
+        success = milter_client_run(client, &error);
+        if (!success) {
+            g_print("%s\n", error->message);
+            g_error_free(error);
+        }
         signal(SIGINT, sigint_handler);
     } else {
         g_print("%s\n", error->message);
