@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -62,8 +62,6 @@ static gdouble writing_timeout = MILTER_SERVER_CONTEXT_DEFAULT_WRITING_TIMEOUT;
 static gdouble reading_timeout = MILTER_SERVER_CONTEXT_DEFAULT_READING_TIMEOUT;
 static gdouble end_of_message_timeout = MILTER_SERVER_CONTEXT_DEFAULT_END_OF_MESSAGE_TIMEOUT;
 
-#define PROGRAM_NAME "milter-test-server"
-
 #define MILTER_TEST_SERVER_ERROR                                \
     (g_quark_from_static_string("milter-test-server-error-quark"))
 
@@ -76,17 +74,19 @@ typedef enum
 
 typedef struct _Message
 {
+    gchar *envelope_from;
+    gchar *original_envelope_from;
+    GList *recipients;
+    GList *original_recipients;
     MilterHeaders *headers;
     MilterHeaders *original_headers;
-    gchar *envelope_from;
-    GList *recipients;
     GString *body_string;
     GString *replaced_body_string;
 } Message;
 
 typedef struct _ProcessData
 {
-    GMainLoop *main_loop;
+    MilterEventLoop *loop;
     GTimer *timer;
     gboolean success;
     guint reply_code;
@@ -159,19 +159,11 @@ set_macro (gpointer key, gpointer value, gpointer user_data)
     }
 }
 
-static void
-send_recipient (MilterServerContext *context, ProcessData *data)
-{
-    milter_server_context_envelope_recipient(context,
-                                             *(recipients + data->current_recipient));
-    data->current_recipient++;
-}
-
 static void
 set_macros_for_connect (MilterProtocolAgent *agent)
 {
     milter_protocol_agent_set_macros(agent, MILTER_COMMAND_CONNECT,
-                                     "{daemon_name}", PROGRAM_NAME,
+                                     "{daemon_name}", g_get_prgname(),
                                      "{if_name}", "localhost",
                                      "{if_addr}", "127.0.0.1",
                                      "j", "milter-test-server",
@@ -261,177 +253,210 @@ set_macros_for_unknown (MilterProtocolAgent *agent)
 {
 }
 
-static void
-send_header (MilterServerContext *context, MilterProtocolAgent *agent, ProcessData *data)
+static gboolean
+send_connect (MilterServerContext *context, ProcessData *data)
 {
+    const gchar *host_name;
 
+    set_macros_for_connect(MILTER_PROTOCOL_AGENT(context));
+    if (milter_option_get_step(data->option) & MILTER_STEP_NO_CONNECT)
+        return FALSE;
+
+    if (connect_host)
+        host_name = connect_host;
+    else
+        host_name = "mx.example.net";
+
+    if (connect_address) {
+        milter_server_context_connect(context,
+                                      host_name,
+                                      connect_address,
+                                      connect_address_length);
+    } else {
+        struct sockaddr_in address;
+        const gchar ip_address[] = "192.168.123.123";
+        uint16_t port = 50443;
+
+        address.sin_family = AF_INET;
+        address.sin_port = g_htons(port);
+        inet_pton(AF_INET, ip_address, &(address.sin_addr));
+        milter_server_context_connect(context,
+                                      host_name,
+                                      (struct sockaddr *)(&address),
+                                      sizeof(address));
+    }
+
+    return TRUE;
+}
+
+static gboolean
+send_helo (MilterServerContext *context, ProcessData *data)
+{
+    set_macros_for_helo(MILTER_PROTOCOL_AGENT(context));
+    if (milter_option_get_step(data->option) & MILTER_STEP_NO_HELO)
+        return FALSE;
+
+    milter_server_context_helo(context, helo_host);
+    return TRUE;
+}
+
+static gboolean
+send_envelope_from (MilterServerContext *context, ProcessData *data)
+{
+    milter_server_context_reset_message_related_data(context);
+    set_macros_for_envelope_from(MILTER_PROTOCOL_AGENT(context));
+    if (milter_option_get_step(data->option) & MILTER_STEP_NO_ENVELOPE_FROM)
+        return FALSE;
+
+    milter_server_context_envelope_from(context, envelope_from);
+    return TRUE;
+}
+
+static gboolean
+send_envelope_recipient (MilterServerContext *context, ProcessData *data)
+{
+    gchar *recipient;
+
+    set_macros_for_envelope_recipient(MILTER_PROTOCOL_AGENT(context));
+    if (milter_option_get_step(data->option) & MILTER_STEP_NO_ENVELOPE_RECIPIENT)
+        return FALSE;
+
+    recipient = recipients[data->current_recipient];
+    if (!recipient)
+        return FALSE;
+
+    milter_server_context_envelope_recipient(context, recipient);
+    data->current_recipient++;
+    return TRUE;
+}
+
+static gboolean
+send_unknown (MilterServerContext *context, ProcessData *data)
+{
+    if (milter_option_get_step(data->option) & MILTER_STEP_NO_UNKNOWN)
+        return FALSE;
+    if (!unknown_command)
+        return FALSE;
+    if (milter_option_get_version(data->option) < 3)
+        return FALSE;
+
+    set_macros_for_unknown(MILTER_PROTOCOL_AGENT(context));
+    milter_server_context_unknown(context, unknown_command);
+    return TRUE;
+}
+
+static gboolean
+send_data (MilterServerContext *context, ProcessData *data)
+{
+    set_macros_for_data(MILTER_PROTOCOL_AGENT(context));
+    if (milter_option_get_step(data->option) & MILTER_STEP_NO_DATA)
+        return FALSE;
+
+    if (milter_option_get_version(data->option) < 4)
+        return FALSE;
+
+    milter_server_context_data(context);
+    return TRUE;
+}
+
+static gboolean
+send_header (MilterServerContext *context, ProcessData *data)
+{
     MilterHeader *header;
 
-    set_macros_for_header(agent);
+    set_macros_for_header(MILTER_PROTOCOL_AGENT(context));
+    if (milter_option_get_step(data->option) & MILTER_STEP_NO_HEADERS)
+        return FALSE;
+    if (milter_headers_length(data->option_headers) == 0)
+        return FALSE;
+
     header = milter_headers_get_nth_header(data->option_headers, 1);
     milter_server_context_header(context, header->name, header->value);
     milter_headers_remove(data->option_headers, header);
+    return TRUE;
 }
 
-static void
-send_body_chunk (MilterServerContext *context, MilterProtocolAgent *agent, ProcessData *data)
+static gboolean
+send_end_of_header (MilterServerContext *context, ProcessData *data)
+{
+    set_macros_for_end_of_header(MILTER_PROTOCOL_AGENT(context));
+    if (milter_option_get_step(data->option) & MILTER_STEP_NO_END_OF_HEADER)
+        return FALSE;
+
+    milter_server_context_end_of_header(context);
+    return TRUE;
+}
+
+static gboolean
+send_body (MilterServerContext *context, ProcessData *data)
 {
     gchar *body_chunk;
 
-    set_macros_for_body(agent);
+    set_macros_for_body(MILTER_PROTOCOL_AGENT(context));
+    if (milter_option_get_step(data->option) & MILTER_STEP_NO_BODY)
+        return FALSE;
 
     body_chunk = data->body_chunks[data->current_body_chunk];
+    if (!body_chunk)
+        return FALSE;
+
     milter_server_context_body(context, body_chunk, strlen(body_chunk));
     data->current_body_chunk++;
+    return TRUE;
+}
+
+static gboolean
+send_end_of_message (MilterServerContext *context, ProcessData *data)
+{
+    set_macros_for_end_of_message(MILTER_PROTOCOL_AGENT(context));
+    milter_server_context_end_of_message(context, NULL, 0);
+    return TRUE;
 }
 
 static void
 cb_continue (MilterServerContext *context, gpointer user_data)
 {
     ProcessData *data = user_data;
-    MilterStepFlags step = MILTER_STEP_NONE;
     MilterProtocolAgent *agent;
 
     agent = MILTER_PROTOCOL_AGENT(context);
 
-    if (data->option)
-        step = milter_option_get_step(data->option);
-
     switch (milter_server_context_get_state(context)) {
     case MILTER_SERVER_CONTEXT_STATE_NEGOTIATE:
-        set_macros_for_connect(agent);
-        if (!(step & MILTER_STEP_NO_CONNECT)) {
-            const gchar *host_name;
-
-            if (connect_host)
-                host_name = connect_host;
-            else
-                host_name = "mx.local.net";
-
-            if (connect_address) {
-                milter_server_context_connect(context,
-                                              host_name,
-                                              connect_address,
-                                              connect_address_length);
-            } else {
-                struct sockaddr_in address;
-                const gchar ip_address[] = "192.168.123.123";
-                uint16_t port = 50443;
-
-                address.sin_family = AF_INET;
-                address.sin_port = g_htons(port);
-                inet_pton(AF_INET, ip_address, &(address.sin_addr));
-                milter_server_context_connect(context,
-                                              host_name,
-                                              (struct sockaddr *)(&address),
-                                              sizeof(address));
-            }
-            if (!(step & MILTER_STEP_NO_REPLY_CONNECT)) {
-                break;
-            }
-        }
+        if (send_connect(context, data))
+            break;
     case MILTER_SERVER_CONTEXT_STATE_CONNECT:
-        set_macros_for_helo(agent);
-        if (!(step & MILTER_STEP_NO_HELO)) {
-            milter_server_context_helo(context, helo_host);
-            if (!(step & MILTER_STEP_NO_REPLY_HELO)) {
-                break;
-            }
-        }
+        if (send_helo(context, data))
+            break;
     case MILTER_SERVER_CONTEXT_STATE_HELO:
-        milter_server_context_reset_message_related_data(context);
-        set_macros_for_envelope_from(agent);
-        if (!(step & MILTER_STEP_NO_ENVELOPE_FROM)) {
-            milter_server_context_envelope_from(context, envelope_from);
-            if (!(step & MILTER_STEP_NO_REPLY_ENVELOPE_FROM)) {
-                break;
-            }
-        }
+        if (send_envelope_from(context, data))
+            break;
     case MILTER_SERVER_CONTEXT_STATE_ENVELOPE_FROM:
-        set_macros_for_envelope_recipient(agent);
-        if (!(step & MILTER_STEP_NO_ENVELOPE_RECIPIENT)) {
-            send_recipient(context, data);
-            if (!(step & MILTER_STEP_NO_REPLY_ENVELOPE_RECIPIENT)) {
-                break;
-            }
-        }
+        if (send_envelope_recipient(context, data))
+            break;
     case MILTER_SERVER_CONTEXT_STATE_ENVELOPE_RECIPIENT:
-        if (!(step & MILTER_STEP_NO_ENVELOPE_RECIPIENT) &&
-            *(recipients + data->current_recipient)) {
-            set_macros_for_envelope_recipient(agent);
-            send_recipient(context, data);
-            if (!(step & MILTER_STEP_NO_REPLY_ENVELOPE_RECIPIENT)) {
-                break;
-            }
-        }
-        if (!(step & MILTER_STEP_NO_UNKNOWN) &&
-            unknown_command &&
-            milter_option_get_version(data->option) >= 3) {
-            set_macros_for_unknown(agent);
-            milter_server_context_unknown(context, unknown_command);
-            if (!(step & MILTER_STEP_NO_REPLY_UNKNOWN)) {
-                break;
-            }
-        }
+        if (send_envelope_recipient(context, data))
+            break;
+        if (send_unknown(context, data))
+            break;
     case MILTER_SERVER_CONTEXT_STATE_UNKNOWN:
-        set_macros_for_data(agent);
-        if (!(step & MILTER_STEP_NO_DATA) &&
-            milter_option_get_version(data->option) >= 4) {
-            milter_server_context_data(context);
-            if (!(step & MILTER_STEP_NO_REPLY_DATA)) {
-                break;
-            }
-        }
+        if (send_data(context, data))
+            break;
     case MILTER_SERVER_CONTEXT_STATE_DATA:
-        set_macros_for_header(agent);
-        if (!(step & MILTER_STEP_NO_HEADERS)) {
-            MilterHeader *header;
-            header = milter_headers_get_nth_header(data->option_headers, 1);
-            milter_server_context_header(context, header->name, header->value);
-            milter_headers_remove(data->option_headers, header);
-            if (!(step & MILTER_STEP_NO_REPLY_HEADER)) {
-                break;
-            }
-        }
+        if (send_header(context, data))
+            break;
     case MILTER_SERVER_CONTEXT_STATE_HEADER:
-        if (!(step & MILTER_STEP_NO_HEADERS) &&
-            milter_headers_length(data->option_headers) > 0) {
-            send_header(context, agent, data);
-            if (!(step & MILTER_STEP_NO_REPLY_HEADER)) {
-                break;
-            }
-
-            while (milter_headers_length(data->option_headers) > 0) {
-                send_header(context, agent, data);
-            }
-        }
-        set_macros_for_end_of_header(agent);
-        if (!(step & MILTER_STEP_NO_END_OF_HEADER)) {
-            milter_server_context_end_of_header(context);
-            if (!(step & MILTER_STEP_NO_REPLY_END_OF_HEADER)) {
-                break;
-            }
-        }
+        if (send_header(context, data))
+            break;
+        if (send_end_of_header(context, data))
+            break;
     case MILTER_SERVER_CONTEXT_STATE_END_OF_HEADER:
-        if (!(step & MILTER_STEP_NO_BODY)) {
-            send_body_chunk(context, agent, data);
-            if (!(step & MILTER_STEP_NO_REPLY_BODY)) {
-                break;
-            }
-        }
+        if (send_body(context, data))
+            break;
     case MILTER_SERVER_CONTEXT_STATE_BODY:
-        if (!(step & MILTER_STEP_NO_BODY) &&
-            data->body_chunks[data->current_body_chunk]) {
-            send_body_chunk(context, agent, data);
-            if (!(step & MILTER_STEP_NO_REPLY_BODY)) {
-                break;
-            }
-            while(data->body_chunks[data->current_body_chunk]) {
-                send_body_chunk(context, agent, data);
-            }
-        }
-        set_macros_for_end_of_message(agent);
-        milter_server_context_end_of_message(context, NULL, 0);
+        if (send_body(context, data))
+            break;
+        send_end_of_message(context, data);
         break;
     case MILTER_SERVER_CONTEXT_STATE_END_OF_MESSAGE:
         send_quit(context, data);
@@ -588,6 +613,17 @@ cb_delete_header (MilterServerContext *context,
     milter_headers_delete_header(data->message->headers, name, index);
 }
 
+static gchar *
+normalize_envelope_address (const gchar *address)
+{
+    if (g_str_has_prefix(address, "<") &&
+        g_str_has_suffix(address, ">")) {
+        return g_strdup(address);
+    } else {
+        return g_strdup_printf("<%s>", address);
+    }
+}
+
 static void
 cb_change_from (MilterServerContext *context,
                 const gchar *from, const gchar *parameters,
@@ -597,8 +633,7 @@ cb_change_from (MilterServerContext *context,
 
     if (data->message->envelope_from)
         g_free(data->message->envelope_from);
-    data->message->envelope_from = g_strdup(from);
-    milter_headers_change_header(data->message->headers, "From", 1, from);
+    data->message->envelope_from = normalize_envelope_address(from);
 }
 
 static void
@@ -607,23 +642,10 @@ cb_add_recipient (MilterServerContext *context,
                   gpointer user_data)
 {
     ProcessData *data = user_data;
-    MilterHeader *header;
-    gchar *old_value;
-    
-    data->message->recipients = g_list_append(data->message->recipients,
-                                              g_strdup(recipient));
-    header = milter_headers_lookup_by_name(data->message->headers, "To");
-    if (!header) {
-        milter_headers_add_header(data->message->headers, "To", recipient);
-        return;
-    }
 
-    old_value = header->value;
-    if (strlen(header->value) == 0)
-        header->value = g_strdup(recipient);
-    else
-        header->value = g_strdup_printf("%s, %s", header->value, recipient);
-    g_free(old_value);
+    data->message->recipients =
+        g_list_append(data->message->recipients,
+                      normalize_envelope_address(recipient));
 }
 
 static void
@@ -631,40 +653,11 @@ cb_delete_recipient (MilterServerContext *context, const gchar *recipient,
                      gpointer user_data)
 {
     ProcessData *data = user_data;
-    MilterHeader *header;
-    gchar *old_value;
+    gchar *normalized_recipient;
 
-    remove_recipient(&(data->message->recipients), recipient);
-
-    header = milter_headers_lookup_by_name(data->message->headers, "To");
-    if (!header)
-        return;
-
-    if (strstr(header->value, recipient)) {
-        gchar **recipients, **new_recipients;
-        gint i, n_recipients, new_pos;
-
-        recipients = g_strsplit(header->value, ", ", -1);
-        n_recipients = g_strv_length(recipients);
-        new_recipients = g_new0(gchar*, n_recipients);
-
-        for (i = 0, new_pos = 0; i < n_recipients; i++) {
-            if (!strcmp(recipients[i], recipient))
-                continue;
-            new_recipients[new_pos] = recipients[i];
-            new_pos++;
-        }
-        new_recipients[n_recipients - 1] = NULL;
-
-        g_free(recipients);
-        old_value = header->value;
-        if (new_pos > 0)
-            header->value = g_strjoinv(", ", new_recipients);
-        else
-            header->value = g_strdup("");
-        g_strfreev(new_recipients);
-        g_free(old_value);
-    }
+    normalized_recipient = normalize_envelope_address(recipient);
+    remove_recipient(&(data->message->recipients), normalized_recipient);
+    g_free(normalized_recipient);
 }
 
 static void
@@ -733,7 +726,7 @@ cb_finished (MilterFinishedEmittable *emittable, gpointer user_data)
     ProcessData *data = user_data;
 
     g_timer_stop(data->timer);
-    g_main_loop_quit(data->main_loop);
+    milter_event_loop_quit(data->loop);
 }
 
 static void
@@ -782,6 +775,76 @@ cb_end_of_message_timeout (MilterServerContext *context, gpointer user_data)
     send_abort(context, data);
 }
 
+static void
+cb_state_transited (MilterServerContext *context,
+                    MilterServerContextState state,
+                    gpointer user_data)
+{
+    ProcessData *data = user_data;
+    MilterStepFlags step = MILTER_STEP_NONE;
+
+    if (data->option)
+        step = milter_option_get_step(data->option);
+
+    switch (state) {
+    case MILTER_SERVER_CONTEXT_STATE_CONNECT:
+        if (!(step & MILTER_STEP_NO_REPLY_CONNECT))
+            break;
+        if (send_helo(context, data))
+            break;
+    case MILTER_SERVER_CONTEXT_STATE_HELO:
+        if (!(step & MILTER_STEP_NO_REPLY_HELO))
+            break;
+        if (send_envelope_from(context, data))
+            break;
+    case MILTER_SERVER_CONTEXT_STATE_ENVELOPE_FROM:
+        if (!(step & MILTER_STEP_NO_REPLY_ENVELOPE_FROM))
+            break;
+        if (send_envelope_recipient(context, data))
+            break;
+    case MILTER_SERVER_CONTEXT_STATE_ENVELOPE_RECIPIENT:
+        if (!(step & MILTER_STEP_NO_REPLY_ENVELOPE_RECIPIENT))
+            break;
+        if (send_envelope_recipient(context, data))
+            break;
+        if (send_unknown(context, data))
+            break;
+        if (send_data(context, data))
+            break;
+    case MILTER_SERVER_CONTEXT_STATE_UNKNOWN:
+        if (!(step & MILTER_STEP_NO_REPLY_UNKNOWN))
+            break;
+        if (send_data(context, data))
+            break;
+    case MILTER_SERVER_CONTEXT_STATE_DATA:
+        if (!(step & MILTER_STEP_NO_REPLY_DATA))
+            break;
+        if (send_header(context, data))
+            break;
+    case MILTER_SERVER_CONTEXT_STATE_HEADER:
+        if (!(step & MILTER_STEP_NO_REPLY_HEADER))
+            break;
+        if (send_header(context, data))
+            break;
+        if (send_end_of_header(context, data))
+            break;
+    case MILTER_SERVER_CONTEXT_STATE_END_OF_HEADER:
+        if (!(step & MILTER_STEP_NO_REPLY_END_OF_HEADER))
+            break;
+        if (send_body(context, data))
+            break;
+    case MILTER_SERVER_CONTEXT_STATE_BODY:
+        if (!(step & MILTER_STEP_NO_REPLY_BODY))
+            break;
+        if (send_body(context, data))
+            break;
+        if (send_end_of_message(context, data))
+            break;
+    default:
+        break;
+    }
+}
+
 static void
 setup (MilterServerContext *context, ProcessData *data)
 {
@@ -816,6 +879,8 @@ setup (MilterServerContext *context, ProcessData *data)
     CONNECT(reading_timeout);
     CONNECT(end_of_message_timeout);
 
+    CONNECT(state_transited);
+
 #undef CONNECT
 }
 
@@ -875,7 +940,17 @@ cb_connection_error (MilterErrorEmittable *emittable, GError *error, gpointer us
 
     data->success = FALSE;
     data->error = g_error_copy(error);
-    g_main_loop_quit(data->main_loop);
+    milter_event_loop_quit(data->loop);
+}
+
+static gboolean
+set_name (const gchar *option_name,
+          const gchar *value,
+          gpointer data,
+          GError **error)
+{
+    g_set_prgname(value);
+    return TRUE;
 }
 
 static gboolean
@@ -884,7 +959,7 @@ print_version (const gchar *option_name,
                gpointer data,
                GError **error)
 {
-    g_printf("%s %s\n", PROGRAM_NAME, VERSION);
+    g_print("%s %s\n", g_get_prgname(), VERSION);
     exit(EXIT_SUCCESS);
     return TRUE;
 }
@@ -1084,6 +1159,7 @@ parse_mail_contents (const gchar *contents, GError **error)
     gchar **lines, **first_lines;
     GList *recipient_list = NULL;
     GString *body_string;
+    GPtrArray *chunks;
 
     lines = g_strsplit(contents, "\n", -1);
     first_lines = lines;
@@ -1114,10 +1190,25 @@ parse_mail_contents (const gchar *contents, GError **error)
         }
     }
 
+    chunks = g_ptr_array_new();
     body_string = g_string_new(NULL);
     for (; *lines; lines++) {
-        g_string_append_printf(body_string, "%s\r\n", *lines);
+        gsize chunk_length;
+        chunk_length = body_string->len + strlen(*lines) + strlen("\r\n");
+        if (chunk_length > MILTER_CHUNK_SIZE) {
+            g_ptr_array_add(chunks, g_strdup(body_string->str));
+            g_string_truncate(body_string, 0);
+        }
+        g_string_append(body_string, *lines);
+        g_string_append(body_string, "\r\n");
     }
+    if (body_string->len > 0) {
+        g_ptr_array_add(chunks,
+                        g_strndup(body_string->str,
+                                  body_string->len - strlen("\r\n")));
+    }
+    g_string_free(body_string, TRUE);
+    body_chunks = (gchar **)g_ptr_array_free(chunks, FALSE);
 
     if (recipient_list) {
         gint i, length;
@@ -1132,14 +1223,6 @@ parse_mail_contents (const gchar *contents, GError **error)
         g_list_free(recipient_list);
     }
 
-    if (body_string->len > 0) {
-        g_string_truncate(body_string, body_string->len - strlen("\r\n"));
-        body_chunks = g_new0(gchar*, 2);
-        body_chunks[0] = g_strdup(body_string->str);
-        body_chunks[1] = NULL;
-    }
-    g_string_free(body_string, TRUE);
-
     g_strfreev(first_lines);
 
     return TRUE;
@@ -1230,6 +1313,8 @@ parse_color_arg (const gchar *option_name, const gchar *value,
 
 static const GOptionEntry option_entries[] =
 {
+    {"name", 0, 0, G_OPTION_ARG_CALLBACK, set_name,
+     N_("Use NAME as the milter server name"), "NAME"},
     {"connection-spec", 's', 0, G_OPTION_ARG_CALLBACK, parse_spec_arg,
      N_("The spec of client socket. "
         "(unix:PATH|inet:PORT[@HOST]|inet6:PORT[@HOST])"),
@@ -1302,8 +1387,15 @@ message_new (void)
 
     message = g_new0(Message, 1);
     message->envelope_from = g_strdup(envelope_from);
-    for (i = 0; i < g_strv_length(recipients); i++)
-        message->recipients = g_list_append(message->recipients, g_strdup(recipients[i]));
+    message->original_envelope_from = g_strdup(envelope_from);
+    message->recipients = NULL;
+    message->original_recipients = NULL;
+    for (i = 0; i < g_strv_length(recipients); i++) {
+        message->recipients = \
+            g_list_append(message->recipients, g_strdup(recipients[i]));
+        message->original_recipients = \
+            g_list_append(message->original_recipients, g_strdup(recipients[i]));
+    }
     message->headers = milter_headers_copy(option_headers);
     message->original_headers = milter_headers_copy(option_headers);
 
@@ -1320,10 +1412,16 @@ free_message (Message *message)
 {
     if (message->envelope_from)
         g_free(message->envelope_from);
+    if (message->original_envelope_from)
+        g_free(message->original_envelope_from);
     if (message->recipients) {
         g_list_foreach(message->recipients, (GFunc)g_free, NULL);
         g_list_free(message->recipients);
     }
+    if (message->original_recipients) {
+        g_list_foreach(message->original_recipients, (GFunc)g_free, NULL);
+        g_list_free(message->original_recipients);
+    }
     if (message->headers)
         g_object_unref(message->headers);
     if (message->original_headers)
@@ -1337,7 +1435,7 @@ free_message (Message *message)
 static void
 init_process_data (ProcessData *data)
 {
-    data->main_loop = g_main_loop_new(NULL, FALSE);
+    data->loop = milter_glib_event_loop_new(NULL);
     data->timer = g_timer_new();
     data->success = TRUE;
     data->quarantine_reason = NULL;
@@ -1356,6 +1454,8 @@ init_process_data (ProcessData *data)
 static void
 free_process_data (ProcessData *data)
 {
+    if (data->loop)
+        g_object_unref(data->loop);
     if (data->timer)
         g_timer_destroy(data->timer);
     if (data->option)
@@ -1394,16 +1494,33 @@ post_option_parse (GOptionContext *option_context,
                    gpointer data,
                    GError **error)
 {
+    guint i, n_recipients;
+    gchar **normalized_recipients;
+
     if (!helo_host)
         helo_host = g_strdup("delian");
+
     if (!envelope_from)
-        envelope_from = g_strdup("<kou+send@example.com>");
+        envelope_from = "sender@example.com";
+    envelope_from = normalize_envelope_address(envelope_from);
+
     if (!recipients)
-        recipients = g_strsplit("<kou+receive@example.com>", ",", -1);
+        recipients = g_strsplit("receiver@example.org", ",", -1);
+    n_recipients = g_strv_length(recipients);
+    normalized_recipients = g_new0(gchar *, n_recipients + 1);
+    for (i = 0; i < n_recipients; i++) {
+        normalized_recipients[i] = normalize_envelope_address(recipients[i]);
+    }
+    normalized_recipients[i] = NULL;
+    g_strfreev(recipients);
+    recipients = normalized_recipients;
+
     if (!milter_headers_lookup_by_name(option_headers, "From"))
         milter_headers_add_header(option_headers, "From", envelope_from);
+
     if (!milter_headers_lookup_by_name(option_headers, "To"))
         milter_headers_add_header(option_headers, "To", recipients[0]);
+
     if (!body_chunks) {
         const gchar body[] =
             "La de da de da 1.\n"
@@ -1423,6 +1540,95 @@ free_option_values (void)
         g_object_unref(option_headers);
 }
 
+static void
+print_envelope_item (const gchar *prefix, const gchar *label,
+                     const gchar *address)
+{
+    const gchar *color_starter = NO_COLOR;
+    const gchar *color_finisher = NO_COLOR;
+
+    if (use_color) {
+        if (g_str_has_prefix(prefix, "+")) {
+            color_starter = GREEN_COLOR;
+            color_finisher = NORMAL_COLOR;
+        } else if (g_str_has_prefix(prefix, "-")) {
+            color_starter = RED_COLOR;
+            color_finisher = NORMAL_COLOR;
+        };
+    }
+
+    g_print("%s%s %s:%s%s\n",
+            color_starter,
+            prefix, label, address,
+            color_finisher);
+}
+
+static void
+print_envelope_from (Message *message)
+{
+    const gchar *label = "MAIL FROM";
+    const gchar *original_envelope_from = message->original_envelope_from;
+    const gchar *result_envelope_from = message->envelope_from;
+
+    if (milter_utils_strcmp0(original_envelope_from,
+                             result_envelope_from) == 0) {
+        print_envelope_item(" ", label, original_envelope_from);
+    } else {
+        print_envelope_item("-", label, original_envelope_from);
+        print_envelope_item("+", label, result_envelope_from);
+    }
+}
+
+static void
+print_envelope_recipients (Message *message)
+{
+    const gchar *label = "RCPT TO";
+    const GList *original_node, *result_node;
+
+    original_node = message->original_recipients;
+    result_node = message->recipients;
+    while (original_node && result_node) {
+        const gchar *original_recipient = original_node->data;
+        const gchar *result_recipient = result_node->data;
+
+        if (milter_utils_strcmp0(original_recipient, result_recipient) == 0) {
+            print_envelope_item(" ", label, original_recipient);
+            result_node = g_list_next(result_node);
+            original_node = g_list_next(original_node);
+        } else if (milter_utils_strcmp0(original_recipient, result_recipient) != 0) {
+            print_envelope_item("-", label, original_recipient);
+            print_envelope_item("+", label, result_recipient);
+            result_node = g_list_next(result_node);
+            original_node = g_list_next(original_node);
+        } else if (!g_list_find_custom((GList*)result_node,
+                                       original_recipient,
+                                       (GCompareFunc)milter_utils_strcmp0)){
+            print_envelope_item("-", label, original_recipient);
+            original_node = g_list_next(original_node);
+        } else {
+            print_envelope_item("+", label, result_recipient);
+            result_node = g_list_next(result_node);
+        }
+    }
+
+    for (; original_node; original_node = g_list_next(original_node)) {
+        const gchar *original_recipient = original_node->data;
+        print_envelope_item("-", label, original_recipient);
+    }
+
+    for (; result_node; result_node = g_list_next(result_node)) {
+        const gchar *result_recipient = result_node->data;
+        print_envelope_item("+", label, result_recipient);
+    }
+}
+
+static void
+print_envelope (Message *message)
+{
+    print_envelope_from(message);
+    print_envelope_recipients(message);
+}
+
 static void
 print_header (const gchar *prefix, MilterHeader *header)
 {
@@ -1431,38 +1637,38 @@ print_header (const gchar *prefix, MilterHeader *header)
     const gchar *color_finisher = NO_COLOR;
 
     if (use_color) {
-        if (g_str_has_prefix(prefix, "+"))
+        if (g_str_has_prefix(prefix, "+")) {
             color_starter = GREEN_COLOR;
-        else if (g_str_has_prefix(prefix, "-"))
+            color_finisher = NORMAL_COLOR;
+        } else if (g_str_has_prefix(prefix, "-")) {
             color_starter = RED_COLOR;
-        else
-            color_starter = NORMAL_COLOR;
-        color_finisher = NORMAL_COLOR;
+            color_finisher = NORMAL_COLOR;
+        }
     }
 
     value_lines = g_strsplit(header->value, "\n", -1);
     first_line = value_lines;
 
     if (!*value_lines) {
-        g_printf("%s%s %s:%s\n",
-                 color_starter,
-                 prefix, header->name,
-                 color_finisher);
+        g_print("%s%s %s:%s\n",
+                color_starter,
+                prefix, header->name,
+                color_finisher);
         g_strfreev(value_lines);
         return;
     }
 
-    g_printf("%s%s %s: %s%s\n",
-             color_starter,
-             prefix, header->name, *value_lines,
-             color_finisher);
+    g_print("%s%s %s: %s%s\n",
+            color_starter,
+            prefix, header->name, *value_lines,
+            color_finisher);
     value_lines++;
 
     while (*value_lines) {
-        g_printf("%s%s %s%s\n",
-                 color_starter,
-                 prefix, *value_lines,
-                 color_finisher);
+        g_print("%s%s %s%s\n",
+                color_starter,
+                prefix, *value_lines,
+                color_finisher);
         value_lines++;
     }
 
@@ -1485,16 +1691,21 @@ print_headers (Message *message)
             print_header(" ", original_header);
             result_node = g_list_next(result_node);
             node = g_list_next(node);
-        } else if (g_str_equal(original_header->name, result_header->name)){
+        } else if (!g_list_find_custom((GList*)node,
+                                       result_header,
+                                       (GCompareFunc)milter_header_compare)) {
+            print_header("+", result_header);
+            result_node = g_list_next(result_node);
+        } else if (!g_list_find_custom((GList*)result_node,
+                                       original_header,
+                                       (GCompareFunc)milter_header_compare)) {
+            print_header("-", original_header);
+            node = g_list_next(node);
+        } else if (g_str_equal(original_header->name, result_header->name)) {
             print_header("-", original_header);
             print_header("+", result_header);
             result_node = g_list_next(result_node);
             node = g_list_next(node);
-        } else if (!g_list_find_custom((GList*)result_node,
-                                       original_header,
-                                       (GCompareFunc)milter_header_compare)){
-            print_header("-", original_header);
-            node = g_list_next(node);
         } else {
             print_header("+", result_header);
             result_node = g_list_next(result_node);
@@ -1523,7 +1734,7 @@ get_charset (const gchar *value)
         return NULL;
 
     pos += strlen("charset=");
-    
+
     if (pos[0] == '"' || pos[0] == '\'') {
         end_pos = strchr(pos + 1, pos[0]);
         if (!end_pos)
@@ -1580,22 +1791,25 @@ print_body (Message *message)
                                     &bytes_written,
                                     &error);
         if (!translated_body)
-            g_printf("%s\n", body_string);
+            g_print("%s\n", body_string);
         else
-            g_printf("%s\n", translated_body);
+            g_print("%s\n", translated_body);
 
         if (charset)
             g_free(charset);
     } else {
-        g_printf("%s\n", body_string);
+        g_print("%s\n", body_string);
     }
 }
 
 static void
 print_message (Message *message)
 {
+    g_print("Envelope:------------------------------\n");
+    print_envelope(message);
+    g_print("Header:--------------------------------\n");
     print_headers(message);
-    g_printf("---------------------------------------\n");
+    g_print("Body:----------------------------------\n");
     print_body(message);
 }
 
@@ -1618,36 +1832,35 @@ print_status (MilterServerContext *context, ProcessData *data)
         g_type_class_unref(enum_class);
     }
 
-    g_printf("status: %s\n", status_name);
+    g_print("status: %s\n", status_name);
 }
 
 static void
 print_result (MilterServerContext *context, ProcessData *data)
 {
     if (data->error) {
-        g_printf("%s\n", data->error->message);
+        g_print("%s\n", data->error->message);
         return;
     }
 
     print_status(context, data);
+    g_print("elapsed-time: %g seconds\n", g_timer_elapsed(data->timer, NULL));
     if (data->quarantine_reason) {
-        g_printf("The message was quarantined.: %s\n",
-                 data->quarantine_reason);
+        g_print("Quarantine reason: <%s>\n", data->quarantine_reason);
     }
 
     if (output_message) {
-        g_printf("\n");
+        g_print("\n");
         print_message(data->message);
-        g_printf("\n");
     }
-
-    g_printf("elapsed-time: %g seconds\n", g_timer_elapsed(data->timer, NULL));
 }
 
 static void
 setup_context (MilterServerContext *context, ProcessData *process_data)
 {
-    milter_server_context_set_name(context, PROGRAM_NAME);
+    milter_agent_set_event_loop(MILTER_AGENT(context), process_data->loop);
+
+    milter_server_context_set_name(context, g_get_prgname());
 
     milter_server_context_set_connection_timeout(context, connection_timeout);
     milter_server_context_set_reading_timeout(context, reading_timeout);
@@ -1670,12 +1883,12 @@ start_process (MilterServerContext *context, ProcessData *process_data)
         success = milter_server_context_establish_connection(context, &error);
 
     if (!success) {
-        g_printf("%s\n", error->message);
+        g_print("%s\n", error->message);
         g_error_free(error);
         return FALSE;
     }
 
-    g_main_loop_run(process_data->main_loop);
+    milter_event_loop_run(process_data->loop);
 
     print_result(context, process_data);
 
@@ -1718,7 +1931,7 @@ main (int argc, char *argv[])
                                    post_option_parse);
 
     if (!g_option_context_parse(option_context, &argc, &argv, &error)) {
-        g_printf("%s\n", error->message);
+        g_print("%s\n", error->message);
         g_error_free(error);
         g_option_context_free(option_context);
         free_option_values();
@@ -1733,7 +1946,7 @@ main (int argc, char *argv[])
         ProcessData *process_data;
         gint i;
 
-        threads = g_new0(GThread*, n_threads);
+        threads = g_new0(GThread *, n_threads);
         process_data = g_new0(ProcessData, n_threads);
         for (i = 0; i < n_threads; i++) {
             init_process_data(&process_data[i]);
